



# Notes:
#
#   1. You cannot base logic on whether a current MV tag exists in the code below.  It will always exist, because before this gets called, all missing MVs
#      will get added to the projection.
#
#   2. To trigger this, create a projection, say on the client, save, and grab it's projection ID.  Call GetModvars to get the library ID and put that into the appropriate
#      environment variable. Then call OpenProjection. Make sure to leave the projection on the client open while doing this because the proj ID changes when you open/close.
# def PC_UpdateModvars(projection, created_projection):

#     if PC_COST_INPUT_RECORDS_TAG_V1 in projection:
#         _convert_cost_input_records_1(projection, PC_COST_INPUT_RECORDS_TAG_V1)
#         _convert_cost_input_records_2(projection, pcmvt.PC_COST_INPUT_RECORDS_TAG)

#     elif PC_COST_INPUT_RECORDS_TAG_V2 in projection:
#         _convert_cost_input_records_2(projection, PC_COST_INPUT_RECORDS_TAG_V2)

#     if PC_COSTING_SECT_RECORDS_TAG_V1 in projection:
#         _convert_costing_sect_records_1(projection, PC_COSTING_SECT_RECORDS_TAG_V1)

#     if PC_ACT_RECORDS_TAG_V1 in projection:
#         _convert_act_records_1(projection, PC_ACT_RECORDS_TAG_V1)

#     # delete tags
#     for tag in PC_TagsForDeletion:
#         if tag in projection:
#             del projection[tag]

#     return projection

# PC_COST_INPUT_RECORDS_TAG_V1    = '<PC_CostInputRecords_V1>'
# PC_COST_INPUT_RECORDS_TAG_V2    = '<PC_CostInputRecords_V2>'
# PC_COSTING_SECT_RECORDS_TAG_V1  = '<PC_CostingSectionRecords_V1>'
# PC_ACT_RECORDS_TAG_V1           = '<PC_ActivityRecords_V1>'

# PC_TagsForDeletion = [
#     PC_COST_INPUT_RECORDS_TAG_V1,
#     PC_COST_INPUT_RECORDS_TAG_V2,
#     PC_COSTING_SECT_RECORDS_TAG_V1,
#     PC_ACT_RECORDS_TAG_V1,
# ]

# def _reload_string_constants_cost_inputs(cost_input_records_value):
#     cost_input_DB = pcrd.PC_load_cost_input_DB()

#     # Create a dict out of the database for fast lookups. Key-value pairs are input ID - inner dicts where the inner dicts are
#     # cost input string constant IDs and unit of measure string constant IDs
#     cost_input_DB_dict = {}

#     for cost_input_DB_obj in cost_input_DB:
#         cost_input_DB_dict[str(cost_input_DB_obj[pcdbk.PC_COST_INPUT_ID_KEY_CIDB])] = {
#             'costInputStringConstant'     : cost_input_DB_obj[pcdbk.PC_COST_INPUT_STR_CONST_KEY_CIDB],
#             'unitOfMeasureStringConstant' : cost_input_DB_obj[pcdbk.PC_UNIT_OF_MEASURE_STR_CONST_KEY_CIDB],
#         }

#     for rec in cost_input_records_value:
#         cost_input_ID = rec[pcmvf.PC_COST_INPUT_ID]

#         if cost_input_ID in cost_input_DB_dict:
#             rec[pcmvf.PC_COST_INPUT_STR_CONST] = cost_input_DB_dict[cost_input_ID]['costInputStringConstant']
#             rec[pcmvf.PC_COST_INPUT_UNIT_OF_MEASURE_STR_CONST] = cost_input_DB_dict[cost_input_ID]['unitOfMeasureStringConstant']

# def _reload_string_constants_cost_sections_and_subsections(cost_section_records_value):
#     # Both come from the same database.
#     cost_section_DB = pcrd.PC_load_costing_sect_DB()

#     ##########   Costing sections   ###########

#     # Create a dict out of the database for fast lookups. Key-value pairs are keys described below and costing string constants.
#     cost_sections_DB_dict = {}

#     for cost_section_DB_obj in cost_section_DB:
#         # Costing section rows will have a subsection ID of -1
#         costing_subsect_ID = cost_section_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_CSDB]

#         if costing_subsect_ID == -1:
#             # Costing sections require area type ID, area ID, and costing section ID to positively identify them.
#             key = (str(
#                 str(cost_section_DB_obj[pcdbk.PC_AREA_TYPE_ID_KEY_CSDB])
#                 + '_' + str(cost_section_DB_obj[pcdbk.PC_AREA_ID_KEY_CSDB])
#                 + '_' + str(cost_section_DB_obj[pcdbk.PC_COSTING_SECT_ID_KEY_CSDB])
#             ))
#             cost_sections_DB_dict[key] = cost_section_DB_obj[pcdbk.PC_COSTING_SECT_STR_CONST_KEY_CSDB]

#     for rec in cost_section_records_value:
#         area_type_ID = rec[pcmvf.PC_COSTING_SECT_AREA_TYPE_ID]
#         area_ID = rec[pcmvf.PC_COSTING_SECT_AREA_ID]
#         costing_sect_ID = rec[pcmvf.PC_COSTING_SECT_ID]

#         key = str(str(area_type_ID) + '_' + str(area_ID) + '_' + str(costing_sect_ID))

#         if key in cost_sections_DB_dict:
#             rec[pcmvf.PC_COSTING_SECT_STR_CONST] = cost_sections_DB_dict[key]

#     ##########   Costing subsections   ###########

#     cost_subsection_DB_dict = {}

#     for cost_subsection_DB_obj in cost_section_DB:
#         # Costing subsection rows will have a subsection ID other than -1
#         costing_subsect_ID = cost_subsection_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_CSDB]

#         if costing_subsect_ID != -1:
#             # Costing sections require area type ID, area ID, and costing section ID to positively identify them. Costing subsections require these in addition to
#             # the costing subsection ID.
#             key = (str(
#                 str(cost_subsection_DB_obj[pcdbk.PC_AREA_TYPE_ID_KEY_CSDB])
#                 + '_' + str(cost_subsection_DB_obj[pcdbk.PC_AREA_ID_KEY_CSDB])
#                 + '_' + str(cost_subsection_DB_obj[pcdbk.PC_COSTING_SECT_ID_KEY_CSDB])
#                 + '_' + str(cost_subsection_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_CSDB])
#             ))
#             cost_subsection_DB_dict[key] = cost_subsection_DB_obj[pcdbk.PC_COSTING_SUBSECT_STR_CONST_KEY_CSDB]

#     for section_rec in cost_section_records_value:
#         area_type_ID = section_rec[pcmvf.PC_COSTING_SECT_AREA_TYPE_ID]
#         area_ID = section_rec[pcmvf.PC_COSTING_SECT_AREA_ID]
#         costing_sect_ID = section_rec[pcmvf.PC_COSTING_SECT_ID]
#         subsect_records = section_rec[pcmvf.PC_COSTING_SECT_SUBSECT_RECORDS]

#         for subsect_rec in subsect_records:
#             subsect_ID = subsect_rec[pcmvf.PC_COSTING_SUBSECT_ID]

#             key = str(str(area_type_ID) + '_' + str(area_ID) + '_' + str(costing_sect_ID) + '_' + str(subsect_ID))

#             if key in cost_subsection_DB_dict:
#                 subsect_rec[pcmvf.PC_COSTING_SUBSECT_STR_CONST] = cost_subsection_DB_dict[key]

# def _convert_cost_input_records_1(projection, cost_input_records_tag):
#     cost_input_records_value = projection[cost_input_records_tag]

#     # String constants were added.

#     for rec in cost_input_records_value:
#         if not (pcmvf.PC_COST_INPUT_STR_CONST in rec):
#             rec[pcmvf.PC_COST_INPUT_STR_CONST] = ''

#         if not (pcmvf.PC_COST_INPUT_UNIT_OF_MEASURE_STR_CONST in rec):
#             rec[pcmvf.PC_COST_INPUT_UNIT_OF_MEASURE_STR_CONST] = ''

#     if cost_input_records_tag != pcmvt.PC_COST_INPUT_RECORDS_TAG:
#         projection[pcmvt.PC_COST_INPUT_RECORDS_TAG] = deepcopy(cost_input_records_value)

# def _convert_cost_input_records_2(projection, cost_input_records_tag):
#     cost_input_records_value = projection[cost_input_records_tag]

#     _reload_string_constants_cost_inputs(cost_input_records_value)

#     if cost_input_records_tag != pcmvt.PC_COST_INPUT_RECORDS_TAG:
#         projection[pcmvt.PC_COST_INPUT_RECORDS_TAG] = deepcopy(cost_input_records_value)

# def _convert_act_records_1(projection, act_records_tag):
#     act_records_value = projection[act_records_tag]

#     for rec in act_records_value:
#         if not (pcmvf.PC_ACT_STR_CONST in rec):
#             rec[pcmvf.PC_ACT_STR_CONST] = ''

#     if act_records_tag != pcmvt.PC_ACT_RECORDS_TAG:
#         projection[pcmvt.PC_ACT_RECORDS_TAG] = deepcopy(act_records_value)

# def _convert_costing_sect_records_1(projection, costing_sect_records_tag):
#     cost_section_records_value = projection[costing_sect_records_tag]

#     _reload_string_constants_cost_sections_and_subsections(cost_section_records_value)

#     if costing_sect_records_tag != pcmvt.PC_COSTING_SECT_RECORDS_TAG:
#         projection[pcmvt.PC_COSTING_SECT_RECORDS_TAG] = deepcopy(cost_section_records_value)



#////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#/////                                                                                                          /////
#/////                                          Input Modvars                                                   /////
#/////                                                                                                          /////
#////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#/* Subnational Wizard */
CS_SubnatWizProjTag                      = '<CS_SubnatWizProj_V1>'
CS_SubnatRegionTag                       = '<CS_SubnatRegion_V1>'
CS_FirstTimeEnteringCoverageTag          = '<CS_FirstTimeEnteringCoverage_V1>'
CS_SubnatDefDataMstIDsetTag              = '<CS_SubnatDefDataMstIDset_V1>'
CS_SubnatDataSourceArrayTag              = '<CS_SubnatDataSourceArray_V1>'

CS_DataSourceTag                         = '<CS_DataSource_V1>'

# Meta data
CS_IDMapTag                              = '<CS_IDMap_V1>'
CS_FinalIndicesTag                       = '<CS_FinalIndices_V1>'

#/* Intervention info */
CS_NumIntervsTag                         = '<CS_NumIntervs_V1>'
CS_IVInfoTag                             = '<CS_IVInfo_V1>'
CS_IVGroupInfoTag                        = '<CS_IVGroupInfo_V1>'
CS_IVSubgroupInfoTag                     = '<CS_IVSubgroupInfo_V1>'

#/* General */
CS_DevModeTag                            = '<CS_DevMode_V1>'
CS_FirstYearTag                          = '<CS_FirstYear_V1>'
CS_FinalYearTag                          = '<CS_FinalYear_V1>'
CS_ProjValidTag                          = '<CS_ProjValid_V1>'
CS_ProjValidDateTag                      = '<CS_ProjValidDateTime_V1>'
CS_DefaultDBChoiceTag                    = '<CS_DefaultDBChoice_V1>'

#/* Controls */
CS_CovBYTag                              = '<CS_CovBY_V1>'
CS_MenARecommendedTag                    = '<CS_MenARecommended_V1>'
CS_IPTpCBCheckedTag                      = '<CS_IPTpCBStatus_V1>'
CS_HaveCalculatedChDeathsByCauseTag      = '<CS_HaveCalculatedChDeathsByCause_V1>'
CS_FirstYearAtWorkTag                    = '<CS_FirstYearAtWork_V1>'
CS_LastYearAtWorkTag                     = '<CS_LastYearAtWork_V1>'
CS_AbortCalcFamPlanTag                   = '<CS_AbortCalcFamPlan_V1>'
CS_CovEntryModeTag                       = '<CS_CovEntryMode_V1>'
CS_CalcCovFromInputsTag                  = '<CS_CalcCovFromInputs_V1>'
CS_EnterBFImproveDataTag                 = '<CS_BFEntryMode_V1>'
CS_BFPromoCBsTag                         = '<CS_BFPromoCBs_V1>'
CS_StuntMatrixOnTag                      = '<CS_StuntingMatrixOn_V1>'
CS_WasteMatrixOnTag                      = '<CS_WastingMatrixOn_V1>'
CS_UseDetailedStuntMatrixTag             = '<CS_UseDetailedStuntMatrix_V1>'
CS_UseDetailedWasteMatrixTag             = '<CS_UseDetailedWasteMatrix_V1>'
CS_FertilityRisksMatrixOnTag             = '<CS_FertilityRisksMatrixOn_V1>'
CS_DirectEntryFITag                      = '<CS_DirectEntryFI_V1>'
CS_SelectedIVSetTag                      = '<CS_SelectedIVSet_V1>'

#/* Health status */
CS_NutrDeficTag                          = '<CS_NutrDefic_V1>'
CS_VitADefInPWTag                        = '<CS_VitADefInPW_V1>'
CS_StatusAtBirthTag                      = '<CS_StatusAtBirth_V1>'
CS_DiarrIncTag                           = '<CS_DiarrInc_V1>'
CS_SevereDiarrIncTag                     = '<CS_SevereDiarrInc_V1>'
CS_PneumIncTag                           = '<CS_PneumInc_V1>'
CS_MeninIncTag                           = '<CS_MeninInc_V1>'
CS_MaternalAnemiaTag                     = '<CS_MaternalAnemia_V1>'
CS_PercWom15t49LowBMITag                 = '<CS_PercWwom15t49LowBMI_V1>'
CS_SyphilisAmongWRATag                   = '<CS_SyphilisAmongWRA_V1>'
CS_PercExposedFalciparumTag              = '<CS_PercExposedFalciparum_V1>'
CS_PercWomenWithPrevPreEclampTag         = '<CS_PercWomenWithPrevPreEclamp_V1>'
CS_PercWomenWithChronicHypertensionTag   = '<CS_PercWomenWithChronicHypertension_V1>'
CS_FolicAcidDeficPrevTag                 = '<CS_FolicAcidDeficPrev_V1>'
CS_CongenitalDueToNTDsAFTag              = '<CS_CongenitalDueToNTDsAF_V1>'
CS_StuntDistrTag                         = '<CS_StuntDistr_V1>'
CS_WasteDistrTag                         = '<CS_WasteDistr_V1>'
CS_SingleStuntDistrTag                   = '<CS_SingleStuntDistr_V1>'
CS_SingleWasteDistrTag                   = '<CS_SingleWasteDistr_V1>'
CS_DiarrPathTag	                         = '<CS_DiarrPath_V1>'
CS_PneumPathTag                          = '<CS_PneumPath_V1>'
CS_MeninPathTag	                         = '<CS_MeninPath_V1>'
CS_BaselineMortTag                       = '<CS_BaselineMort_V1>'
CS_PercDthByCauseTag	                 = '<CS_PercDthByCause_V1>'
CS_AdolMortRatioTag                      = '<CS_AdolMortRatio_V1>'
CS_PercAdolDeathsByCauseTag              = '<CS_PercAdolDeathsByCause_V1>'  
CS_MatMortRatioTag                       = '<CS_MatMortRatio_V1>'
CS_PercMatDthByCauseTag                  = '<CS_PercMatDthByCause_V1>'
CS_PercPregEndAbortTag                   = '<CS_PercPregEndAbort_V1>'
CS_AbortIncRatioTag                      = '<CS_AbortIncRatio_V1>'
CS_SBRateTag                             = '<CS_SBRate_V1>'
CS_SBCausesTag                           = '<CS_SBCauses_V1>'
CS_PerCapitaIncTag                       = '<CS_PerCapitaInc_V1>'
CS_FoodInsecureTag                       = '<CS_FoodInsecure_V1>'
CS_AvgHouseholdSizeTag	                 = '<CS_AverageHouseholdSize_V1>'
CS_IncludeCostForBCRTag                  = '<CS_IncludeCostForBCR_V1>'
CS_IncludeBenefitForBCRTag               = '<CS_IncludeBenefitForBCR_V1>'
CS_BaselineGDPTag                        = '<CS_BaselineGDP_V1>'
CS_AnnualPercGrowthRateTag	             = '<CS_GNIPerCap_V1>'
CS_LaborForceParRateTag	                 = '<CS_LaborForceParRate_V1>'
CS_FemaleLaborForceParRateTag	         = '<CS_FemaleLaborForceParRate_V1>'
CS_PercGDPAllocWageTag	                 = '<CS_LaborShareInc_V1>'
CS_DiscountRatesTag                      = '<CS_DiscountRates_V1>'
CS_FactorsTag                            = '<CS_Factors_V1>'

#/* Coverage */
CS_CoverageTag                           = '<CS_Coverage_V1>'
CS_BFPrevTag                             = '<CS_BFPrev_V1>'
CS_EarlyInitBFPrevTag                    = '<CS_EarlyInitBFPrev_V1>'
CS_RoutDetVaccCovTag                     = '<CS_RoutDetVaccCov_V1>'
CS_PentaDetVaccCovTag                    = '<CS_PentaDetVaccCov_V1>'
CS_VaccineSuppPercTag                    = '<CS_VaccineSuppPerc_V1>'
CS_AgeAndBirthOrderMatrixTag	         = '<CS_AgeAndBirthOrderMatrix_V1>'
CS_BirthIntervalsMatrixTag               = '<CS_BirthIntervalsMatrix_V1>'

#/* Maternal, Stillbirth, and Child Efficacy */
CS_MatEffTag                             = '<CS_MatEff_V1>'
CS_SBEffTag                              = '<CS_SBEff_V1>'
CS_ChildEffTag                           = '<CS_ChildEff_V1>'
CS_DetVaccEffTag                         = '<CS_DetVaccEff_V1>'

#/* Effectiveness of interventions on incidence */
CS_DiarrIncReductionTag                  = '<CS_DiarrIncReduct_V1>'
CS_SevDiarrIncReductionTag               = '<CS_SevereDiarrIncReduct_V1>'
CS_PneumIncReductionTag                  = '<CS_PneumIncReduct_V1>'
CS_MeninIncReductionTag                  = '<CS_MeninIncReduct_V1>'
CS_RelRisksDiarrTag                      = '<CS_RelRisksDiarr_V1>'
CS_RelRisksPneumTag                      = '<CS_RelRisksPneum_V1>'
CS_RelRisksMeninTag	                     = '<CS_RelRisksMenin_V1>'

#/* Herd effectiveness of vaccines */
CS_HerdVaccEffTag                        = '<CS_HerdVaccEff_V1>'

#/* Impact on food security */
CS_ImpOnFoodSecurityTag                  = '<CS_ImpOnFoodSecurity_V1>'

#/* Effectiveness of nutrition interventions */
CS_AgeAndBirthOrderTag                   = '<CS_AgeAndBirthOrder_V1>'
CS_BirthIntervalsTag	                 = '<CS_BirthIntervals_V1>'
CS_IntervForBOTag                        = '<CS_IntervForBO_V1>'
CS_ImpactOnStuntTag                      = '<CS_ImpactOnStunt_V1>'
CS_ImpactOnWasteTag                      = '<CS_ImpactOnWaste_V1>'
CS_ImpactOnMatAnemiaTag  	             = '<CS_ImpactOnMatAnemia_V1>'
CS_ImpactBFPromoTag                      = '<CS_ImpactBFPromo_V1>'
CS_KMConBFTag                            = '<CS_KMConBF_V1>'
CS_LBWFTag                               = '<CS_LBWF_V1>'
CS_ReducStuntTag                         = '<CS_ReducStunt_V1>'
CS_EducAttainTag                         = '<CS_EducAttain_V1>'

#/* Impact of under-nutrition on mortality */
CS_RelRisksStuntTag                      = '<CS_RelRisksStunt_V1>'
CS_RelRisksWasteTag                      = '<CS_RelRisksWaste_V1>'
CS_RelRisksIUGR_LBWTag	                 = '<CS_RelRisksIUGRLBW_V1>'
CS_ImpactBFOnMortNeoNatTag               = '<CS_ImpactBFOnMortNeoNat_V1>'
CS_ImpactBFOnMortPostNatTag              = '<CS_ImpactBFOnMort_V1>'
CS_RelRisksAnemiaTag                     = '<CS_RelRisksAnemia_V1>'

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                         Result Modvars                                                   /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CS_PercDeathsAvtdSeqPTag                 = '<CS_PercDeathsAvtdSeqP_V1>'
CS_MortalityRatesTag                     = '<CS_MortalityRates_V1>'
CS_DistPathDiarIncResultTag              = '<CS_DistPathDiarIncResult_V1>'
  
# // Neonatal, child sectors
CS_ChDeathsTag                           = '<CS_ChDeaths_V1>'
CS_ChDeathsByYearTag                     = '<CS_ChDeathsByYear_V1>'
CS_ChDeathsPrevdTag                      = '<CS_ChDeathsPrevd_V1>'
CS_ChDeathsByCauseTag                    = '<CS_ChDeathsByCause_V1>'
CS_DeathsByCauseByBirthCohortTag         = '<CS_ChDeathsByCauseByBirthCohort_V1>'
CS_ChDeathsPrevdByCauseTag               = '<CS_ChDeathsPrevdByCause_V1>'
CS_ChDeathsPrevdByIntTag                 = '<CS_ChDeathsPrevdByInt_V1>'
CS_ChDeathsPrevByInterByCauseTag         = '<CS_ChDeathsPrevByIntByCause_V1>'
CS_DetVaccCovResultsTag                  = '<CS_DetVaccCovResults_V1>'
CS_DAvtdByVaccByCohortTag                = '<CS_DAvtdByVaccByCohort_V1>'
CS_DAvtdByVaccByCohortByCauseTag	     = '<CS_DAvtdByVaccByCohortByCause_V1>'

#// Maternal sector
CS_MatDeathsTag                          = '<CS_MatDeaths_V1>'
CS_MatDeathsPrevTag                      = '<CS_MatDeathsPrev_V1>'
CS_MatDeathsByCauseTag                   = '<CS_MatDeathsByCause_V1>'
CS_MatDeathsPrevByCauseTag               = '<CS_MatDeathsPrevByCause_V1>'
CS_MatDeathsPrevByInterTag               = '<CS_MatDeathsPrevByInter_V1>'
CS_MatDeathsPrevByInterByCauseTag	     = '<CS_MatDeathsPrevByInterByCause_V1>'
CS_NumAbortionsTag	                     = '<CS_NumAbortions_V1>'
CS_PercPregAnemicWomenTag                = '<CS_PercPregAnemicWomen_V1>'

#// Stillbirth sector
CS_SBResultsTag                          = '<CS_SBResults_V1>'
CS_SBPrevResultsTag                      = '<CS_SBPrevResults_V1>'
CS_SBByCauseResultsTag                   = '<CS_SBByCauseResults_V1>'
CS_SBPrevByCauseResultsTag               = '<CS_SBPrevByCauseResults_V1>'
CS_SBPrevByInterResultsTag               = '<CS_SBPrevByInterResults_V1>'
CS_SBPrevByInterByCauseTag               = '<CS_SBPrevByInterByCause_V1>'

#// Mortality rates sector
CS_SBRateResultTag                       = '<CS_SBRateResult_V1>'
CS_MatMortRatioResultTag                 = '<CS_MatMortRatioResult_V1>'
CS_MatMortRateResultTag                  = '<CS_MatMortRateResult_V1>'
CS_MortalityRateResultTag                = '<CS_MortRateResult_V1>'
CS_RedMortIntervResultTag                = '<CS_RedMortIntervResult_V1>'
CS_RedMortIntervSortedIDsTag             = '<CS_RedMortIntervSortedIDs_V1>'

#// Nutrition sector
CS_ShutDownBOAttriByIVTag                = '<CS_ShutDownBOAttriByIV_V1>'
CS_SubOpBOAvertByInterTag	             = '<CS_SuboptimalBOAvertByInter_V1>'
CS_MAM_PINTag                            = '<CS_MAM_PIN_V1>'
CS_SAM_PINTag                            = '<CS_SAM_PIN_V1>'
CS_PInVarHtForAgeStuntTag	             = '<CS_PInVarHtForAgeStunt_V1>'
CS_PInVarWtForHtWasteTag                 = '<CS_PInVarWtForHtWaste_V1>'
CS_PInVarWtForHtWasteCFTag               = '<CS_PInVarWtForHtWasteCF_V1>'
CS_NumStuntedTag                         = '<CS_NumStunted_V1>'
CS_NumStuntedAvertedTag	                 = '<CS_NumStuntedAverted_V1>'
CS_NumStuntedAvertedByInterTag	         = '<CS_NumStuntedAvertedByInter_V1>'
CS_PStuntRedAccountByIntBaseYrTag        = '<CS_PStuntRedAccountByIntBaseYr_V1>'
CS_AdditionalYrsSchoolingGainedTag	     = '<CS_AdditionalYrsSchoolingGained_V1>'
CS_PercIncEarnStuntDeclineTag	         = '<CS_PercIncEarnStuntDecline_V1>'
CS_ImpLifeEarnDueToStuntReducTag         = '<CS_ImpLifeEarnDueToStuntReduc_V1>'
CS_LifetimeEarningsObtainedTag	         = '<CS_LifetimeEarningsObtained_V1>'
CS_BFPrevResultTag                       = '<CS_BFPrevResult_V1>'
CS_BirthTermResultTag	                 = '<CS_BirthTermResult_V1>'
CS_PercAnemicWomenTag                    = '<CS_PercAnemicWomen_V1>'
CS_NumAnemicWomenTag                     = '<CS_NumAnemicWomen_V1>'
CS_NumAnemiaCasesPrevTag                 = '<CS_NumAnemiaCasesPrev_V1>'
CS_PrevFoodInsecurityTag                 = '<CS_PrevFoodInsecurity_V1>'

#// Incidence and etiology sector
CS_DiarrIncResultTag                     = '<CS_DiarrIncResult_V1>'
CS_SevereDiarrIncResultTag               = '<CS_SevereDiarrIncRatesResult_V1>'
CS_PneumIncResultTag                     = '<CS_PneumIncResult_V1>'
CS_MeninIncResultTag	                 = '<CS_MeninIncResult_V1>'
CS_NumDiarrCasesTag                      = '<CS_NumDiarrCases_V1>'
CS_NumSevereDiarrCasesTag                = '<CS_NumSevereDiarrCases_V1>'
CS_NumPneumCasesTag                      = '<CS_NumPneumCases_V1>'
CS_NumMeninCasesTag                      = '<CS_NumMeninCases_V1>'
CS_DiarrCasesAvertByInterTag             = '<CS_DiarrCasesAvertByInter_V1>'
CS_SevereDiarrCasesAvertByInterTag       = '<CS_NumSevereDiarrCasesAvertedByInter_V1>'
CS_PneumCasesAvertByInterTag             = '<CS_PneumCasesAvertByInter_V1>'
CS_MeninCasesAvertByInterTag             = '<CS_MeninCasesAvertByInter_V1>'
CS_PercDiarrDeathsByPathTag	             = '<CS_PercDiarrMortByPathResult_V1>'
CS_PercPneumDeathsByPathTag              = '<CS_PercPneumMortByPathResult_V1>'
CS_PercMeninDeathsByPathTag              = '<CS_PercMeninMortByPathResult_V1>'
CS_DiarrDeathsByPathTag                  = '<CS_DiarrDeathsByPath_V1>'
CS_PneumDeathsByPathTag                  = '<CS_PneumDeathsByPath_V1>'
CS_MeninDeathsByPathTag                  = '<CS_MeninDeathsByPath_V1>'
CS_DiarrDeathsByPathBy1yrAgeGrpsTag      = '<CS_DiarrDeathsByPathBy1yrAgeGrps_V1>'
CS_PneumDeathsByPathBy1yrAgeGrpsTag      = '<CS_PneumDeathsByPathBy1yrAgeGrps_V1>'
CS_MeninDeathsByPathBy1yrAgeGrpsTag      = '<CS_MeninDeathsByPathBy1yrAgeGrps_V1>'
CS_PercDiarrCasesByPathTag	             = '<CS_PercDiarrCasesByPath_V1>'
CS_PercSevDiarrCasesByPathTag	         = '<CS_PercSevDiarrCasesByPath_V1>'
CS_PercPneumCasesByPathTag               = '<CS_PercPneumCasesByPath_V1>'
CS_PercMeninCasesByPathTag               = '<CS_PercMeninCasesByPath_V1>'
CS_DiarrCasesByPathTag                   = '<CS_DiarrCasesByPath_V1>'
CS_SevereDiarrCasesByPathTag             = '<CS_SevereDiarCasesByPath_V1>'
CS_PneumCasesByPathTag                   = '<CS_PneumCasesByPath_V1>'
CS_MeninCasesByPathTag                   = '<CS_MeninCasesByPath_V1>'
CS_DiarrCasesByPathBy1YrAgeGrpsTag       = '<CS_DiarrCasesByPathBy1YrAgeGrps_V1>'
CS_SevereDiarrCasesByPathBy1YrAgeGrpsTag = '<CS_SevereDiarrCasesByPathBy1YrAgeGrps_V1>'
CS_PneumCasesByPathBy1YrAgeGrpsTag       = '<CS_PneumCasesByPathBy1YrAgeGrps_V1>'
CS_MeninCasesByPathBy1YrAgeGrpsTag       = '<CS_MeninCasesByPathBy1YrAgeGrps_V1>'
CS_DiarrCasesByBirthCohTag               = '<CS_DiarrCasesByBirthCoh_V1>'
CS_PneumCasesByBirthCohTag               = '<CS_PneumCasesByBirthCoh_V1>'
CS_MeninCasesByBirthCohTag               = '<CS_MeninCasesByBirthCoh_V1>'
CS_SevDiarrCasesByBirthCohTag            = '<CS_SevDiarrCasesByBirthCoh_V1>'
CS_DiarCasesAvtdByYr1stDoseVaccTag       = '<CS_DiarrCasesAvtdByYr1stDoseVacc_V1>'
CS_PneumCasesAvtdByYr1stDoseVaccTag      = '<CS_PneumCasesAvtdByYr1stDoseVacc_V1>'
CS_MeninCasesAvtdByYr1stDoseVaccTag      = '<CS_MeninCasesAvtdByYr1stDoseVacc_V1>'
CS_SevDiarrCasesAvtdByYr1stDoseVaccTag   = '<CS_SevDiarrCasesAvtdByYr1stDoseVacc_V1>'
CS_DiarrDeathsByPathBirthCohTag          = '<CS_DiarrDeathsByPathBirthCoh_V1>'
CS_PneumDeathsByPathBirthCohTag          = '<CS_PneumDeathsByPathBirthCoh_V1>'
CS_MeninDeathsByPathBirthCohTag          = '<CS_MeninDeathsByPathBirthCoh_V1>'
CS_DiarrCasesByPathBirthCohTag           = '<CS_DiarrCasesByPathBirthCoh_V1>'
CS_PneumCasesByPathBirthCohTag           = '<CS_PneumCasesByPathBirthCoh_V1>'
CS_MeninCasesByPathBirthCohTag           = '<CS_MeninCasesByPathBirthCoh_V1>'
CS_SevDiarrCasesByPathBirthCohTag        = '<CS_SevDiarrCasesByPathBirthCoh_V1>'
CS_DiarrDeathsByPathBirthCohVCTag        = '<CS_DiarrDeathsByPathBirthCohVC_V1>'
CS_PneumDeathsByPathBirthCohVCTag        = '<CS_PneumDeathsByPathBirthCohVC_V1>'
CS_MeninDeathsByPathBirthCohVCTag        = '<CS_MeninDeathsByPathBirthCohVC_V1>'
CS_DiarrCasesByPathBirthCohVCTag         = '<CS_DiarrCasesByPathBirthCohVC_V1>'
CS_PneumCasesByPathBirthCohVCTag         = '<CS_PneumCasesByPathBirthCohVC_V1>'
CS_MeninCasesByPathBirthCohVCTag         = '<CS_MeninCasesByPathBirthCohVC_V1>'
CS_PersonsVaccByIntervTag                = '<CS_PersonsVaccByInterv_V1>'
CS_PersonsVaccByIntervBy1YrAgeGrpsTag    = '<CS_PersonsVaccByIntervBy1YrAgeGrps_V1>'

# BCR       
CS_DiscountedCostsTag                    = '<CS_DiscountedCosts_V1>'
CS_DiscountedSocialBenefitsTag           = '<CS_DiscountedSocialBenefits_V1>'
CS_DiscountedEconBenefitsTag             = '<CS_DiscountedEconomicBenefits_V1>'
CS_IncLifetimeEarningsDueToStuntTag      = '<CS_IncLifetimeEarningsDueToStunt_V1>'

CS_TotalDiscountedCostsTag               = '<CS_TotalDiscountedCosts_V1>'
CS_TotalDiscountedSocialBenefitsTag      = '<CS_TotalDiscountedSocialBenefits_V1>'
CS_TotalDiscountedEconBenefitsTag        = '<CS_TotalDiscountedEconomicBenefits_V1>'
CS_LifetimeEarningsIncTag                = '<CS_LifetimeEarningsInc_V1>'
CS_BCRTag                                = '<CS_BCR_V1>'

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                          Meta Modvars                                                    /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CS_SourcesTag                            = '<CS_Sources_V1>'
CS_DiarrIncMetaTag                       = '<CS_DiarrInc_Meta_V1>'
CS_SevereDiarrIncMetaTag                 = '<CS_SevereDiarrInc_Meta_V1>'
CS_PneumIncMetaTag                       = '<CS_PneumInc_Meta_V1>'
CS_MeninIncMetaTag                       = '<CS_MeninInc_Meta_V1>'
CS_BaselineMortMetaTag                   = '<CS_BaselineMort_Meta_V1>'
CS_PercDthByCauseMetaTag                 = '<CS_PercDthByCause_Meta_V1>'
CS_MatMortRatioMetaTag                   = '<CS_MatMortRatio_Meta_V1>'
CS_PercMatDthByCauseMetaTag              = '<CS_PercMatDthByCause_Meta_V1>'
CS_SBRateMetaTag                         = '<CS_SBRate_Meta_V1>'
CS_SBCausesMetaTag                       = '<CS_SBCauses_Meta_V1>'
CS_CoverageMetaTag                       = '<CS_Coverage_Meta_V1>'
CS_RoutDetVaccCovMetaTag                 = '<CS_RoutDetVaccCov_Meta_V1>'
CS_PentaDetVaccCovMetaTag                = '<CS_PentaDetVaccCov_Meta_V1>'
CS_VaccineSuppPercMetaTag                = '<CS_VaccineSuppPerc_Meta_V1>'
CS_MatEffMetaTag                         = '<CS_MatEff_Meta_V1>'
CS_SBEffMetaTag                          = '<CS_SBEff_Meta_V1>'
CS_ChildEffMetaTag                       = '<CS_ChildEff_Meta_V1>'
CS_DetVaccEffMetaTag                     = '<CS_DetVaccEff_Meta_V1>'
CS_DiarrIncReductMetaTag                 = '<CS_DiarrIncReduct_Meta_V1>'
CS_SevereDiarrIncReductMetaTag           = '<CS_SevereDiarrIncReduct_Meta_V1>'
CS_PneumIncReductMetaTag                 = '<CS_PneumIncReduct_Meta_V1>'
CS_MeninIncReductMetaTag                 = '<CS_MeninIncReduct_Meta_V1>'
CS_RelRisksDiarrMetaTag                  = '<CS_RelRisksDiarr_Meta_V1>'
CS_RelRisksPneumMetaTag                  = '<CS_RelRisksPneum_Meta_V1>'
CS_RelRisksMeninMetaTag                  = '<CS_RelRisksMenin_Meta_V1>'
CS_AgeAndBirthOrderMetaTag               = '<CS_AgeAndBirthOrder_Meta_V1>'
CS_BirthIntervalsMetaTag                 = '<CS_BirthIntervals_Meta_V1>'
CS_IntervForBOMetaTag                    = '<CS_IntervForBO_Meta_V1>'
CS_ImpactOnStuntMetaTag                  = '<CS_ImpactOnStunt_Meta_V1>'
CS_ImpactOnWasteMetaTag                  = '<CS_ImpactOnWaste_Meta_V1>'
CS_ImpactOnMatAnemiaMetaTag              = '<CS_ImpactOnMatAnemia_Meta_V1>'
CS_ImpactBFPromoMetaTag                  = '<CS_ImpactBFPromo_Meta_V1>'
CS_KMConBFMetaTag                        = '<CS_KMConBF_Meta_V1>'
CS_ReducStuntMetaTag                     = '<CS_ReducStunt_Meta_V1>'
CS_EducAttainMetaTag                     = '<CS_EducAttain_Meta_V1>'
CS_RelRisksStuntMetaTag                  = '<CS_RelRisksStunt_Meta_V1>'
CS_RelRisksWasteMetaTag                  = '<CS_RelRisksWaste_Meta_V1>'
CS_RelRisksIUGRLBWMetaTag                = '<CS_RelRisksIUGRLBW_Meta_V1>'
CS_ImpactBFOnMortNeoNatMetaTag           = '<CS_ImpactBFOnMortNeoNat_Meta_V1>'
CS_ImpactBFOnMortMetaTag                 = '<CS_ImpactBFOnMort_Meta_V1>'
CS_RelRisksAnemiaMetaTag                 = '<CS_RelRisksAnemia_Meta_V1>'
CS_DetVaccCovResultsMetaTag              = '<CS_DetVaccCovResults_Meta_V1>'
CS_DAvtdByVaccByCohortMetaTag            = '<CS_DAvtdByVaccByCohort_Meta_V1>'
CS_DAvtdByVaccByCohortByCauseMetaTag     = '<CS_DAvtdByVaccByCohortByCause_Meta_V1>'
CS_DebugTag                              = '<CS_Debug_V1>'
### OLD MODVAR TAGS:

#////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#/////                                                                                                          /////
#/////                                          Input Modvars                                                   /////
#/////                                                                                                          /////
#////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

# #/* LiST */
# #/* General */
# CS_TG_FirstYear_MV2                       = '<First Year MV2>'
# CS_TG_FinalYear_MV2                       = '<Final Year MV2>'
# CS_TG_ProjectionValidDate_MV2	          = '<Projection Valid Date and Time: MV2>'

# #/* Controls */
# CS_TG_CovBY_MV                            = '<CovBY MV>'
# CS_TG_ANCMode_MV                          = '<ANCMode MV>'
# CS_TG_ChildbirthMode_MV                   = '<ChildbirthMode MV>'
# CS_TG_CalcCovFromInputs_MV2               = '<CalcCovFromInputs MV2>'
# CS_TG_EnterBFImproveData_MV2              = '<BFEntryMode MV3>'
# CS_TG_StuntingMatrixOn_MV2                = '<StuntingMatrixOn MV2>'
# CS_TG_WastingMatrixOn_MV2                 = '<WastingMatrixOn MV2>'
# CS_TG_FertilityRisksMatrixOn_MV2          = '<FertilityRisksMatrixOn MV2>'
# CS_TG_IPTpCBChecked_MV2                   = '<IPTpCBStatus MV2>'
# CS_TG_AbortCalcFamPlan_MV2                = '<AbortCalcFamPlan MV2>'
# CS_TG_SelectedIVSet_MV                    = '<Selected intervention set MV>'
# #// CS_TG_InterventionList_MV                 = '<InterventionList MV>'
# #// CS_TG_InterventionGroupList_MV            = '<InterventionGroupList MV>'
# #// CS_TG_InterventionSubGroupList_MV         = '<InterventionSubGroupList MV>'

# #/* Health status */
# CS_TG_MenARecommended                     = '<MenA recommended>'
# CS_TG_NutritionalDeficiencies_MV2         = '<Nutritional Deficiencies MV2>'
# CS_TG_VitADefInPW_MV                      = '<VitA deficiency in PW MV>'
# CS_TG_StatusAtBirth_MV                    = '<Status At Birth MV>'
# CS_TG_DiarInc_MV                          = '<DiarInc MV>'
# CS_TG_SevereDiarInc_MV                    = '<SevereDiarInc MV>'
# CS_TG_PneumInc_MV                         = '<Pneumonia Incidence MV>'
# CS_TG_MeningInc_MV                        = '<Meningitis Incidence MV>'
# CS_TG_MaternalAnemia_MV                   = '<Maternal Anemia MV>'
# CS_TG_RelRisksAnemia_MV2                  = '<RelRisksAnemia MV2>'
# CS_TG_PercWom15t49LowBMI_MV               = '<Percentage of women aged 15-49 with low BMI MV>'
# CS_TG_PercExposedFalciparum_MV            = '<Percent exposed to falciparum>'
# CS_TG_FolicAcidDeficPrev_MV               = '<Folic acid deficient prevalence>'
# CS_TG_CongenitalDueToNTDsAF_MV            = '<CongenitalDueToNTDsAF>'
# CS_TG_StuntingDistr_MV                    = '<Stunting distribution MV>'
# CS_TG_SingleStuntingDistr_MV              = '<Single Stunting distribution MV>'
# CS_TG_WastingDistr_MV                     = '<Wasting distribution MV>'
# CS_TG_SingleWastingDistr_MV               = '<Single Wasting distribution MV>'
# CS_TG_UseDetailedStuntingMatrix_MV        = '<Use Detailed Stunting Matrix MV>'
# CS_TG_UseDetailedWastingMatrix_MV         = '<Use Detailed Wasting Maxtrix MV>'
# CS_TG_DiarPath_MV2	                      = '<Diarrhea pathogens MV2>'
# CS_TG_PneumPath_MV                        = '<Pneumonia pathogens MV>'
# CS_TG_MenPath_MV	                      = '<Meningitis pathogens MV>'
# CS_TG_BaselineMort_MV                     = '<BaselineMort MV>'
# CS_TG_PercDthByCause_MV2	              = '<PercDthByCause MV2>'
# CS_TG_MatMortRatio_MV2                    = '<MatMortRatio MV2>'
# CS_TG_PercMatDeathsByCause_MV             = '<PercMatDeathsByCause MV>'
# CS_TG_PercPregEndAbort_MV2                = '<PercPregEndAbort MV2>'
# CS_TG_AbortIncRatio_MV2                   = '<AbortIncRatio MV 2>'
# CS_TG_SBRate_MV2                          = '<SBRate MV2>'
# CS_TG_SBCauses_MV                         = '<SBCauses MV>'
# CS_TG_PerCapitaInc_MV                     = '<PerCapitaInc MV>'
# CS_TG_FoodInsecure_MV                     = '<FoodInsecure MV>'
# CS_TG_AvgHouseholdSize_MV	              = '<Average Household Size MV>'
# CS_TG_FirstYearAtWork_MV                  = '<First year at work MV>'
# CS_TG_LastYearAtWork_MV                   = '<Last year at work MV>'
# CS_TG_GNIPerCap_MV	                      = '<GNIPerCap MV>'
# CS_TG_LaborForceParRate_MV	              = '<LaborForceParRate MV>'
# CS_TG_LaborShareInc_MV	                  = '<LaborShareInc MV>'

# #/* Coverage */
# CS_TG_Coverage_MV3                        = '<Coverage MV3>'
# CS_TG_BreastfeedPrev_MV                   = '<Breastfeeding Prevalence MV>'
# CS_TG_EarlyInitBFPrev_MV2                 = '<Early Initiation BF prevalence MV2>'
# CS_TG_RoutDetVaccCov_MV3                  = '<RoutDetVaccCov MV3>'
# CS_TG_BreastFeedPromoCB_MV                = '<Breastfeeding Promotion Checkboxes MV>'
# CS_TG_PentavalentDetVaccCov_MV            = '<PentavalentDetVaccCov MV>'
# CS_TG_AgeAndBirthOrderMatrix_MV2	      = '<Age And Birth Order Matrix MV2>'
# CS_TG_BirthIntervalsMatrix_MV2            = '<Birth Intervals Matrix MV2>'

# #/* Maternal, Stillbirth, and Child Efficacy */
# CS_TG_MatEfficacy_MV3                     = '<Maternal Efficacy MV3>'
# CS_TG_SBEfficacy_MV2                      = '<SBEfficacy MV2>'
# CS_TG_Efficacy_MV2                        = '<Efficacy MV2>'
# CS_TG_DetVaccEff_MV2                      = '<DetVaccEff MV2>'

# #/* Effectiveness of interventions on incidence */
# CS_TG_DiarrIncReduction_MV3               = '<DiarrIncReduction MV3>'
# CS_TG_SevDiarrIncReduction_MV             = '<Severe Diarrhea Incidence Reduction>'
# CS_TG_PneumIncReduction_MV                = '<Pneumonia Incidence Reduction MV>'
# CS_TG_MeningIncReduction_MV               = '<Meningitis Incidence Reduction MV>'
# CS_TG_RelRisksDiarrhea_MV2                = '<RelRisksDiarrhea MV2>'
# CS_TG_RelRisksPneumonia_MV2          	  = '<RelRisksPneumonia MV2>'
# CS_TG_RelRisksMeningitis_MV2	          = '<RelRisksMeningitis MV2>'

# #/* Herd effectiveness of vaccines */
# CS_TG_HerdVaccEff_MV3                     = '<Custom herd vaccine efficacy MV 3>'

# #/* Effectiveness of nutrition interventions */
# CS_TG_AgeAndBirthOrder_MV2                = '<Age And Birth Order MV2>'
# CS_TG_BirthIntervals_MV2	              = '<Birth Intervals MV2>'
# CS_TG_IntervForBO_MV7                     = '<Maternal Nutrition MV7>'
# CS_TG_ImpactsOnStunting_MV                = '<Impacts On Stunting MV>'
# CS_TG_ImpactsOnWasting_MV4                = '<Impacts On Wasting MV4>'
# CS_TG_ImpactsOnMatAnemia_MV3  	          = '<Impacts On Maternal Anemia MV3>'
# CS_TG_ImpactsBFPromotion_MV3              = '<ImpactsBFPromotion MV3>'
# CS_TG_KMConBF_MV                          = '<Kangaroo Mother Care on Breastfeeding MV>'
# CS_TG_LBWF_MV2                            = '<Low birth weight factor MV2>'
# CS_TG_ReducStunt_MV                       = '<ReducStunt MV>'
# CS_TG_EducationAttain_MV                  = '<EducationAttain MV>'

# #/* Impact of under-nutrition on mortality */
# CS_TG_RelRisksStunting_MV                 = '<RelRisksStunting MV>'
# CS_TG_RelRisksWasting_MV                  = '<RelRisksWasting MV>'
# CS_TG_ImpactBFOnMortNeoNat_MV             = '<ImpactBFOnMortNeoNat MV>'
# CS_TG_ImpactBFOnMortPostNat_MV            = '<ImpactBFOnMort MV>'
# CS_TG_RelRisksIUGR_LBW_MV	              = '<RelRisksIUGR_LBW MV>'
# CS_TG_RelRisksAnemia_MV2                  = '<RelRisksAnemia MV2>'

# # ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# # /////                                                                                                          /////
# # /////                                         Result Modvars                                                   /////
# # /////                                                                                                          /////
# # ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

# CS_TG_PercDeathsAvtdSeqP_MV2            = '<PercDeathsAvtdSeqP MV2>';
  
# # // Neonatal, child sectors
# CS_TG_ChDeaths_MV2                         = '<ChDeaths MV2>'
# CS_TG_ChDeathsByYear_MV2                   = '<ChDeathsByYear MV2>'
# CS_TG_ChDeathsPrevd_MV2                    = '<ChDeathsPrevd MV2>'
# CS_TG_ChDeathsxCause_MV2                   = '<ChDeathsxCause MV2>'
# CS_TG_DeathsByCauseByBirthCohort_MV2       = '<Cause-specific deaths in children under five years of age by year of birth cohort2>'
# CS_TG_ChDeathsPrevdxCause_MV2              = '<ChDeathsPrevdxCause MV2>'
# CS_TG_ChDeathsPrevdxInt_MV3                = '<ChDeathsPrevdxInt MV3>'
# CS_TG_DeathsPrevByInterByCause_MV3         = '<Child deaths prevented by intervention by cause MV3>'
# CS_TG_DetVaccCovResults_MV3                = '<DetVaccCovResults MV3>'
# CS_TG_DAvtdByVaccByCohortResult_MV2        = '<DAvtdByVaccByCohortResult MV2>'
# CS_TG_DAvtdByVaccByCohortByCause_MV3	   = '<DAvtdByVaccByCohortByCause MV3>'

# #// Maternal sector
# CS_TG_MatDeaths_MV2                        = '<MatDeaths MV2>'
# CS_TG_MatDeathsPrev_MV2                    = '<MatDeathsPrev MV2>'
# CS_TG_MatDeathsByCauseResults_MV2          = '<MatDeathsByCauseResults MV2>'
# CS_TG_MatDeathsPrevByCause_MV2             = '<MatDeathsPrevByCause MV2>'
# CS_TG_MatDeathsPrevByInter_MV3             = '<MatDeathsPrevByInter MV3>'
# CS_TG_MatDeathsPrevByInterByCause_MV3	   = '<Maternal deaths prevented by intervention by cause MV3>'
# CS_TG_NumAbortions_MV2	                   = '<Number of Abortions MV2>'
# CS_TG_PercAnemicPregWomen_MV2              = '<Percent of pregnant women who are anemic MV2>'

# #// Stillbirth sector
# CS_TG_SBResults_MV2                        = '<SBResults MV2>'
# CS_TG_SBPrevResults_MV2                    = '<SBPrevResults MV2>'
# CS_TG_SBByCauseResults_MV2                 = '<SBByCauseResults MV2>'
# CS_TG_SBPrevByCauseResults_MV2             = '<SBPrevByCauseResults MV2>'
# CS_TG_SBPrevByInterResults_MV3             = '<SBPrevByInterResults MV3>'
# CS_TG_SBPrevByInterByCause_MV3             = '<Stillbirths prevented by intervention by cause MV3>'

# #// Mortality rates sector
# CS_TG_SBRateResults_MV2                    = '<SBRateResults MV2>'
# CS_TG_MatMortRatioResult_MV2               = '<MatMortRatioResult MV2>'
# CS_TG_MatMortRateResult_MV2                = '<MatMortRateResult MV2>'
# CS_TG_MortalityRateResults_MV2             = '<MortalityRateResults MV2>'
# CS_TG_RedMortIntervResults_MV3             = '<RedMortIntervResults MV3>'
# CS_TG_RedMortIntervSortedIDs_MV3           = '<RedMortIntervSortedIDs MV3>'

# #// Nutrition sector 
# CS_TG_ShutDownBOAttriByIV_MV            = '<ShutDownBOAttriByIV MV>';
# CS_TG_SubOpBOAvertByInter_MV	          = '<Suboptimal birth outcomes averted by intervention MV>';
# CS_TG_MAM_PIN_MV                           = '<MAM_PIN MV>'
# CS_TG_SAM_PIN_MV                           = '<SAM_PIN MV>'
# CS_TG_PInVarHtForAgeStunt_MV2	           = '<PInVarHtForAgeStunt MV2>'
# CS_TG_PInVarWtForHtWaste_MV2               = '<PInVarWtForHtWaste MV2>'
# CS_TG_NumStunted_MV                        = '<NumStunted MV>'
# CS_TG_NumStuntedAverted_MV	               = '<NumStuntedAverted MV>'
# CS_TG_NumStuntedAvertedByInter_MV	       = '<NumStuntedAvertedByInter MV>'
# CS_TG_PStuntRedAccountByIntBaseYr_MV2      = '<PStuntRedAccountByIntBaseYr MV2>'
# CS_TG_PercIncEarnStuntDecline_MV	      = '<PercIncEarnStuntDecline MV>';
# CS_TG_AdditionalYrsSchoolingGained_MV	  = '<AdditionalYrsSchoolingGained MV>';
# CS_TG_ImpLifeEarnDueToStuntReduc_MV     = '<ImpLifeEarnDueToStuntReduc MV>';   
# CS_TG_LifetimeEarningsObtained_MV	      = '<LifetimeEarningsObtained MV>';    
# CS_TG_BFPrevResult_MV2                     = '<BFPrevResult MV2>'
# CS_TG_BirthTermResult_MV2	               = '<Birth Term Results MV2>'
# CS_TG_PercWomenAnemicResults_MV            = '<Percent of women with anemia MV>'
# CS_TG_NumWomenAnemicResults_MV             = '<Number of women with anemia MV>'
# CS_TG_AnemiaCasesPreventedResults_MV       = '<Number of anemia cases prevented MV>'

# #// Incidence and etiology sector
# CS_TG_DiarIncResult_MV2                    = '<DiarIncResult MV2>'
# CS_TG_SevereDiarIncResult_MV               = '<Severe Diarrhea Incidence Rates Result>'
# CS_TG_PneumIncResult_MV2                   = '<PneumIncResult MV2>'
# CS_TG_MeningIncResult_MV2	               = '<Meningitis Incidence Result MV2>'
# CS_TG_DiarCasesResult_MV2                  = '<Number of diarrhea Cases MV2>'
# CS_TG_SevereDiarCasesResult_MV             = '<Number of Severe Diarrhea Cases Result>'
# CS_TG_PneumCasesResult_MV2                 = '<Number of pneumonia cases MV2>'
# CS_TG_MeningCasesResult_MV2                = '<Number of Meningitis Cases- Result MV2>'
# CS_TG_DiarCasesAvertByInter_MV3            = '<DiarCasesAvertByInter MV3>'
# CS_TG_DiarCasesAvertByInter_MV4            = '<DiarCasesAvertByInter MV4>'
# CS_TG_SevereDiarCasesAvertByInter_MV       = '<Number of Severe Diarrhea Cases Averted by Intervention Result>'
# CS_TG_SevereDiarCasesAvertByInter_MV2      = '<Number of Severe Diarrhea Cases Averted by Intervention Result2>'
# CS_TG_PneumCasesAvertByInter_MV3           = '<PneumCasesAvertByInter MV3>'
# CS_TG_MeninCasesAvertByInter_MV3           = '<MeninCasesAvertByInter MV3>'
# CS_TG_PercDiarDeathsByPath_MV2	           = '<PDiarrMortByPathResult MV2>'
# CS_TG_PercPneumDeathsByPath_MV3            = '<PPneumMortByPathResult MV3>'
# CS_TG_PercMeningDeathsByPath_MV2           = '<PMeningMortByPathResult MV2>'
# CS_TG_DiarDeathsByPath_MV2                 = '<Number of deaths due to pathogens by year of impact - Diarrhea MV2>'
# CS_TG_PneumDeathsByPath_MV2                = '<Number of deaths due to pathogens by year of impact - Pneumonia MV2>'
# CS_TG_MeningDeathsByPath_MV2               = '<Number of deaths due to pathogens by year of impact - Meningitis MV2>'
# CS_TG_DiarDeathsByPathBy1yrAgeGrps_MV      = '<Number of deaths due to pathogens by year of impact by 1 year age groups - Diarrhea MV>'
# CS_TG_PneumDeathsByPathBy1yrAgeGrps_MV     = '<Number of deaths due to pathogens by year of impact by 1 year age groups - Pneumonia MV>'
# CS_TG_MeningDeathsByPathBy1yrAgeGrps_MV    = '<Number of deaths due to pathogens by year of impact by 1 year age groups - Meningitis MV>'
# CS_TG_PercDiarCasesByPath_MV2	           = '<PDiarIncWInfectResult MV2>'
# CS_TG_PercSevDiarCasesByPath_MV	           = '<Percent of 1-59 Month Severe Diarrhea Cases by Pathogen>'
# CS_TG_PercPneumCasesByPath_MV3             = '<PPneumIncWInfectResult MV3>'
# CS_TG_PercMeningCasesByPath_MV2            = '<PMeningIncWInfectResult MV2>'
# CS_TG_DiarCasesByPath_MV2                  = '<Number of cases by pathogen by year of impact - Diarrhea MV2>'
# CS_TG_SevereDiarCasesByPath_MV             = '<Number of Severe Diarrhea Cases by Pathogen by Year of Impact Result>'
# CS_TG_PneumCasesByPath_MV2                 = '<Number of cases by pathogen by year of impact - Pneumonia MV2>'
# CS_TG_MeningCasesByPath_MV2                = '<Number of cases by pathogen by year of impact - Meningitis MV2>'
# CS_TG_DiarCasesByPathBy1YrAgeGrps_MV       = '<Number of cases by pathogen by year of impact by 1 year age groups - Diarrhea MV>'
# CS_TG_SevereDiarCasesByPathBy1YrAgeGrps_MV = '<Number of Severe Diarrhea Cases by Pathogen by Year of Impact by 1 year age groups Result>'
# CS_TG_PneumCasesByPathBy1YrAgeGrps_MV      = '<Number of cases by pathogen by year of impact by 1 year age groups - Pneumonia MV>'
# CS_TG_MeningCasesByPathBy1YrAgeGrps_MV     = '<Number of cases by pathogen by year of impact by 1 year age groups - Meningitis MV>'
# CS_TG_DiarCasesByBirthCoh_MV2              = '<Number of cases by cause by birth cohort - Diarrhea MV2>'
# CS_TG_PneumCasesByBirthCoh_MV2             = '<Number of cases by cause by birth cohort - Pneumonia MV2>'
# CS_TG_MeningCasesByBirthCoh_MV2            = '<Number of cases by cause by birth cohort - Meningitis MV2>'
# CS_TG_SevDiarCasesByBirthCoh_MV            = '<Number of cases by cause by birth cohort - Severe diarrhea MV>'
# CS_TG_DiarCasesAvtdByYr1stDoseVacc_MV2     = '<Cases averted by cause by year that the first dose of a vaccine was received by a child - Diarrhea MV2>'
# CS_TG_PneumCasesAvtdByYr1stDoseVacc_MV2    = '<Cases averted by cause by year that the first dose of a vaccine was received by a child - Pneumonia MV2>'
# CS_TG_MeningCasesAvtdByYr1stDoseVacc_MV2   = '<Cases averted by cause by year that the first dose of a vaccine was received by a child - Meningitis MV2>'
# CS_TG_SevDiarCasesAvtdByYr1stDoseVacc_MV   = '<Cases averted by cause by year that the first dose of a vaccine was received by a child - Severe diarrhea MV>'
# CS_TG_DiarDeathsByPathBirthCoh_MV2         = '<Number of deaths by pathogen - Diarrhea MV2>'
# CS_TG_PneumDeathsByPathBirthCoh_MV2        = '<Number of deaths by pathogen - Pneumonia MV2>'
# CS_TG_MeningDeathsByPathBirthCoh_MV2       = '<Number of deaths by pathogen - Meningitis MV2>'
# CS_TG_DiarCasesByPathBirthCoh_MV2          = '<Number of cases by pathogen - Diarrhea MV2>'
# CS_TG_PneumCasesByPathBirthCoh_MV2         = '<Number of cases by pathogen - Pneumonia MV2>'
# CS_TG_MeningCasesByPathBirthCoh_MV2        = '<Number of cases by pathogen - Meningitis MV2>'
# CS_TG_SevDiarCasesByPathBirthCoh_MV        = '<Number of cases by pathogen - Severe diarrhea MV>'
# CS_TG_DiarDeathsByPathBirthCohVC_MV        = '<Number of deaths by pathogen - Diarrhea - Vaccinated cohort MV>'
# CS_TG_PneumDeathsByPathBirthCohVC_MV       = '<Number of deaths by pathogen - Pneumonia - Vaccinated cohort MV>'
# CS_TG_MeningDeathsByPathBirthCohVC_MV      = '<Number of deaths by pathogen - Meningitis - Vaccinated cohort MV>'
# CS_TG_DiarCasesByPathBirthCohVC_MV         = '<Number of cases by pathogen - Diarrhea - Vaccinated cohort MV>'
# CS_TG_PneumCasesByPathBirthCohVC_MV        = '<Number of cases by pathogen - Pneumonia - Vaccinated cohort MV>'
# CS_TG_MeningCasesByPathBirthCohVC_MV       = '<Number of cases by pathogen - Meningitis - Vaccinated cohort MV>'
# CS_TG_PersonsVaccByInterv_MV2              = '<Number of persons vaccinated by intervention MV2>'
# CS_TG_PersonsVaccByIntervBy1YrAgeGrps_MV   = '<Number of persons vaccinated by intervention by 1 year age groups MV>'
# CS_TG_PersonsVaccByIntervBy1YrAgeGrps_MV2  = '<Number of persons vaccinated by intervention by 1 year age groups MV2>'

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                         DefaultData Tags                                                 /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TG_Efficacy                             = '<Efficacy>'
TG_AgeAndBirthOrder                     = '<Age and Birth Order>'
TG_BirthSpacing                         = '<Birth Spacing>'
TG_IntervForBO                          = '<Interventions for birth outcomes>'
TG_ImpactsStunting                      = '<Impacts on Stunting>'
TG_ImpactWastingOnMort                  = '<Impact Wasting on Mortality>'
TG_ImpactOnAnemia                       = '<Impact on Anemia>'
TG_ReducInStunt                         = '<Reduction in stunting>'
TG_EducAttain                           = '<Education attainment>'
TG_RelRiskAnemia                        = '<Relative Risk Anemia>'
TG_ImpactDiarrInc                       = '<Impact on Diarrhea Incidence>'
TG_ImpactPneumInc                       = '<Impact Pneumonia Incidence>'
TG_ImpactMeninInc                       = '<Impact Meningitis Incidence>'
TG_BFImprovement                        = '<Breastfeeding Improvement>'
TG_RelRiskStunting                      = '<Relative Risk Stunting>'
TG_RelRiskWasting                       = '<Relative Risk Wasting>'
TG_RelRiskMortBF                        = '<Relative Risk Mortality Breastfeeding>'
TG_RelRiskDiarrBF                       = '<Relative Risk Diarrhea Breastfeeding>'
TG_RelRiskPneumBF                       = '<Relative Risk Pneumonia Breastfeeding>'
TG_RelRiskMeninBF                       = '<Relative Risk Meningitis Breastfeeding>'
TG_HerdEfficacy                         = '<Herd Efficacy>'
TG_DetVaccEff                           = '<Detailed Vaccine Efficacy>'

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                        DataByCountry Tags                                                /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TG_DataSource                           = '<DataSource>'
TG_MostRecentSurvey                     = '<Most Recent Survey>'
TG_NutritionalDeficiencies              = '<Nutritional deficiencies>'
TG_VitADefInPW                          = '<VitA deficiency in PW>'
TG_BaseBirthOutcomes                    = '<Baseline birth outcomes>'
TG_IncDiarrhea                          = '<Incidence of diarrhea>'
TG_IncSevereDiarrhea                    = '<Incidence of severe diarrhea>'
TG_IncPneum                             = '<Incidence of pneumonia>'
TG_IncMening                            = '<Incidence of meningitis>'
TG_MaternalAnemia                       = '<Maternal anemia>'
TG_PercWomenSevereAnemia                = '<Percent of women with severe anemia>'
TG_PercExposedFalciparum                = '<Percent exposed to falciparum>'
TG_IPTPRec                              = '<IPTP recommended>'
TG_MenArecommended                      = '<MenA recommended>'
TG_PercWom15t49LowBMI                   = '<Prevalence of low BMI in WRA>'
TG_SyphilisAmongWRA                     = '<Syphilis among WRA>'
TG_StuntingDist                         = '<Stunting Distribution>'
TG_WastingDist                          = '<Wasting Distribution>'
TG_PathSevDiarr                         = '<Pathogen incidence severe diarrhea>'
TG_PathAllDiarr                         = '<Pathogen incidence all diarrhea>'
TG_PathIncPneum                         = '<Pathogen incidence pneumonia>'
TG_PathIncMening                        = '<Pathogen incidence meningitis>'
TG_PathMortDiarr                        = '<Pathogen mortality diarrhea>'
TG_PathMortPneum                        = '<Pathogen mortality pneumonia>'
TG_PathMortMening                       = '<Pathogen mortality meningitis>'
TG_MortalityRate                        = '<Mortality Rate>'
TG_DeathsByCause                        = '<Deaths by cause>'
TG_BaselineAdolMort                     = '<Adolescent mortality rate>'
TG_AdolDeathByCause                     = '<Adolescent deaths by cause>'  
TG_BaselineMatMort                      = '<Baseline maternal mortality>'
TG_MatDeathByCause                      = '<Maternal death by cause>'
TG_PercPregEndInAbort                   = '<% pregnancies ending in abortion>'
TG_AbortIncRatio                        = '<Abortion incidence ratio>'
TG_BaselineSBRate                       = '<Baseline stillbirth rate>'
TG_SBCauses                             = '<Stillbirth causes>'
TG_EconomicStatus                       = '<Economic status>'
TG_GDPPerCapita                         = '<GDPPerCapita>'
TG_GDPPerWorker                         = '<GDPPerWorker>'
TG_GNIPerCap                            = '<GNI per capita>'
TG_LaborForceParRate                    = '<Labor force participation rate>'
TG_FemaleLaborForceParRate              = '<Female labor force participation rate>'
TG_LaborShareOfInc                      = '<Labor share of income>'
TG_EconReturnsToSchool                  = '<Economic Returns to Schooling>'
TG_AvgHouseHoldSize                     = '<Average household size>'
TG_Coverage                             = '<Coverage>'
TG_BreastFeeding                        = '<Breastfeeding>'
TG_EarlyInitBFPrev                      = '<Early initiation BF>'
TG_MaternalAF                           = '<Maternal affected fractions>'
TG_SBAffFractions                       = '<Stillbirth affected fractions>'
TG_AffFractions                         = '<Affected fractions>'
TG_RotavirusEff                         = '<Rotavirus effectiveness>'
TG_RotavirusEffSevDiar                  = '<Rotavirus Impact on Severe Diarrhea Incidence effectiveness>'
TG_RotavirusEffAllDiar                  = '<Rotavirus Impact on All Diarrhea Incidence effectiveness>'
TG_PneumoEffMortPneum                   = '<Pneumococcal effectiveness - mortality pneumonia>'
TG_PneumoEffMortMening                  = '<Pneumococcal effectiveness - mortality meningitis>'
TG_PathogenMortOther                    = '<Pathgen mortality -other>'
TG_PneumoEffMortOther                   = '<Pneumococcal effectiveness - mortality other>'
TG_AffFrac                              = '<Affected fraction for impact of therapeutic feeding on other>'
TG_PneumoEffIncPneum                    = '<Pneumococcal effectiveness - incidence pneumonia>'
TG_PneumoEffIncMening                   = '<Pneumococcal effectiveness - incidence meningitis>'
TG_ImpactBirthOutcomesOnMort            = '<Impact of birth outcomes on mortality>'
TG_AFRiskStuntWasteInfDis               = '<Affected fraction for risk of stunting and Wasting on other due to infectious disease>'
TG_AFRiskStuntWasteOtherPreterm         = '<Affected fraction for risk of stunting and Wasting on other due to preterm>'
TG_AFRiskStuntWasteOtherAsphyxia        = '<Affected fraction for risk of stunting and Wasting on other due to asphyxia>'
TG_AFRiskStuntWasteOtherCongenital      = '<Affected fraction for risk of stunting and Wasting on other due to congenital>'
TG_AFRiskStuntWasteOtherRemainOthers    = '<Affected fraction for risk of stunting and Wasting on other due to remaining others>'
TG_LiSTCostingOtherDirectCost           = '<LiST costing other direct cost>'
TG_LiSTCostingIndirectCost              = '<LiST costing indirect cost>'
TG_LBWConversionAlgorithm               = '<LBW conversion algorithm>'
TG_HerdEff                              = '<Herd Effectiveness>'
TG_AFBalancedEnergySupp                 = '<Affected Fraction -Balanced energy Supplementation>'
TG_AFStopSmokingEdu                     = '<Affected Fraction -Stop smoking education>'
TG_PercWomenWithPrevPreEclamp           = '<Percent of women with previous pre-eclampsia>'
TG_PercWomenWithChronicHypertension     = '<Percent of women with chronic hypertension>'


import math
import numpy

from AvenirCommon.Util import GBRange,  GBEqual
from SpectrumCommon.Modvars.GB.GBUtil import getYearIdx
from SpectrumCommon.Const.GB import *
from SpectrumCommon.Const.DP import *
from SpectrumCommon.Const.AM.AMTags import *
from SpectrumCommon.Const.DP.DPTags import *
from SpectrumCommon.Const.PJ.PJNTags import *
from SpectrumCommon.ServerUtils.FilterUtils import get_all_modvars

from SpectrumCommon.Util.AM.AMUtil import GetAM_HIV, getChildNumRecART

sexMap = numpy.zeros(GB_Female + 1).tolist()
sexMap[GB_Male] = 0
sexMap[GB_Female] = 1      

def singleAgeTo5YearAG(age):
    return min(math.floor(age / 5) + 1, DP_MAX_AGE)

def runDPDataChecker(projID, dataYear):
    # dataYear = 2023

    modvars = get_all_modvars(projID)

    firstYear = modvars[PJN_FirstYearTag]
    finalYear = modvars[PJN_FinalYearTag]

    yearIdx = getYearIdx(dataYear, firstYear)
    
    DDC = {
        'PMTCT' : False,
        'ARTPed' : False,
        'ARTMale' : False,
        'ARTFemale' : False,
        'KOSChild' : False,
        'KOSMale' : False,
        'KOSFemale' : False,
        'VSChild' : False,
        'VSMale' : False,
        'VSFemale' : False,
        'ChildARTInterrupt' : False,
        'AdultARTInterrupt' : False,
        'ChildProgression' : False,
        'ChildDistribution' : False,
        'ChildMort' : False,
        'ChildARTMort' : False,
        'AdultProg' : False,
        'AdultDist' : False,
        'AdultMort' : False,
        'AdultARTMort' : False,
        'FRRCD4' : False,
        'FRRAge' : False,
        'CdArt' : False,
        'MTCTRate' : False,
        'AllocDef' : False,
        'Shiny90Valid' : False,
        'UAValid' : False,
        'AdultARTCov' : False,
        'ChildARTCov' : False,
        'PMTCTCov' : False,
        'EPPMaxAdj' : False,
    }

    # year check
    if (firstYear >= dataYear) or (finalYear <= dataYear):
        return DDC

    ARVRegimen = modvars[AM_ARVRegimenTag] 
    DDC['PMTCT'] = bool((ARVRegimen[DP_PrenatalProphylaxis, DP_TripleARTBefPreg, DP_Number, yearIdx] +
        ARVRegimen[DP_PrenatalProphylaxis, DP_TripleARTDurPreg, DP_Number, yearIdx] +
        ARVRegimen[DP_PrenatalProphylaxis, DP_TripleARTDurPreg_Late, DP_Number, yearIdx]) > 100)


    ChildTreatInputs = modvars[AM_ChildTreatInputsTag] 
    ChildARTByAgeGroupPerNum = modvars[AM_ChildARTByAgeGroupPerNumTag]
    if (ChildTreatInputs[DP_PerChildHIVRecART, yearIdx] != DPNotAvail):
        DDC['ARTPed'] = bool(((ChildTreatInputs[DP_PerChildHIVRecART, yearIdx] == DP_Number) and
            (ChildTreatInputs[DP_PerChildHIVRecART, yearIdx] > 0)))
    else:
        DDC['ARTPed'] = bool((ChildTreatInputs[DP_PerChildHIVRecART0_4, yearIdx] +
            ChildTreatInputs[DP_PerChildHIVRecART5_9, yearIdx] +
            ChildTreatInputs[DP_PerChildHIVRecART10_14, yearIdx]) > 0)


    HAARTBySex = modvars[AM_HAARTBySexTag]  
    HAARTBySexPerNum = modvars[AM_HAARTBySexPerNumTag]
    DDC['ARTMale'] = bool((HAARTBySexPerNum[GB_Male, yearIdx] == DP_Number) and (HAARTBySex[GB_Male, yearIdx] > 0))
    DDC['ARTFemale'] = bool((HAARTBySexPerNum[GB_Female, yearIdx] == DP_Number) and (HAARTBySex[GB_Female, yearIdx] > 0))


    KnowledgeOfStatusInput = modvars[AM_KnowledgeOfStatusInputTag]  
    DDC['KOSChild'] = bool(KnowledgeOfStatusInput[DP_Cas_AG_Child, yearIdx] > 0)

    DDC['KOSMale'] = bool(KnowledgeOfStatusInput[DP_Cas_AG_AdultMale, yearIdx] > 0)

    DDC['KOSFemale'] = bool(KnowledgeOfStatusInput[DP_Cas_AG_AdultFemale, yearIdx] > 0)


    ViralSuppressionInput = modvars[AM_ViralSuppressionInputTag] 
    DDC['VSChild'] = bool(((ViralSuppressionInput[DP_Cas_AG_Child, DP_VS_NumTested, yearIdx] >= 0) and 
        (ViralSuppressionInput[DP_Cas_AG_Child, DP_VS_PLHIVSuppressed, yearIdx] >= 0)))

    DDC['VSMale'] = bool(((ViralSuppressionInput[DP_Cas_AG_AdultMale, DP_VS_NumTested, yearIdx] >= 0) and 
        (ViralSuppressionInput[DP_Cas_AG_AdultMale, DP_VS_PLHIVSuppressed, yearIdx] >= 0))) 
    
    DDC['VSFemale'] = bool((ViralSuppressionInput[DP_Cas_AG_AdultFemale, DP_VS_NumTested, yearIdx] >= 0) and 
        (ViralSuppressionInput[DP_Cas_AG_AdultFemale, DP_VS_PLHIVSuppressed, yearIdx] >= 0))
    
    # DDC['ChildARTInterrupt'] = TODO
    # DDC['AdultARTInterrupt'] = TODO
   
    ChildAnnRateProgressLowerCD4 = modvars[AM_ChildAnnRateProgressLowerCD4Tag] 
    DDC['ChildProgression'] = True
    for sd in GBRange(DP_CD4_Per_GT30, DP_CD4_Per_5_10):
        for s in GBRange(GB_Male, GB_Female):
            for a in GBRange(DP_A0t2, DP_A3t4):
                if not GBEqual(ChildAnnRateProgressLowerCD4[DP_Data, s, a, sd], ChildAnnRateProgressLowerCD4[DP_Default, s, a, sd]):
                    DDC['ChildProgression'] = False
    

    ChildDistNewInfectionsCD4 = modvars[AM_ChildDistNewInfectionsCD4Tag] 
    DDC['ChildDistribution'] = True
    for c in GBRange(DP_CD4_Per_GT30, DP_CD4_Per_LT5):
        if not GBEqual(ChildDistNewInfectionsCD4[DP_Data, c], ChildDistNewInfectionsCD4[DP_Default, c]):
            DDC['ChildDistribution'] = False

    ChildMortByCD4NoART = modvars[AM_ChildMortByCD4NoARTTag] 
    DDC['ChildMort'] = True
    for a in GBRange(DP_A0t2, DP_A5t14):
        for b in GBRange(DP_P_Perinatal, DP_P_BF12):
            for c in GBRange(DP_CD4_Per_GT30, DP_CD4_Per_LT5):
                if not GBEqual(ChildMortByCD4NoART[DP_Data, a, b, c], ChildMortByCD4NoART[DP_Default, a, b, c]):
                    DDC['ChildMort'] = False


    DDC['ChildARTMort'] = True
    ChildMortByCD4WithART0to6 = modvars[AM_ChildMortByCD4WithART0to6Tag] 
    ChildMortByCD4WithART7to12 = modvars[AM_ChildMortByCD4WithART7to12Tag] 
    ChildMortByCD4WithARTGT12 = modvars[AM_ChildMortByCD4WithARTGT12Tag] 
    for s in GBRange(GB_Male, GB_Female):
        for a in GBRange(DP_A0, DP_A3t4):
            for c in GBRange(DP_CD4_Per_GT30, DP_CD4_Per_LT5):
                if not GBEqual(ChildMortByCD4WithART0to6[DP_Data, s, a, c], ChildMortByCD4WithART0to6[DP_Default, s, a, c]):
                    DDC['ChildARTMort'] = False
                if not GBEqual(ChildMortByCD4WithART7to12[DP_Data, s, a, c], ChildMortByCD4WithART7to12[DP_Default, s, a, c]):
                    DDC['ChildARTMort'] = False
                if not GBEqual(ChildMortByCD4WithARTGT12[DP_Data, s, a, c], ChildMortByCD4WithARTGT12[DP_Default, s, a, c]):
                    DDC['ChildARTMort'] = False

    ChildMortalityRates = modvars[AM_ChildMortalityRatesTag] 
    for a in GBRange(DP_CD4_0t4, DP_CD4_5t14):
        for b in GBRange(DP_MortRates_LT12Mo, DP_MortRates_GT12Mo):
            for t in GBRange(getYearIdx(firstYear, firstYear), getYearIdx(finalYear, firstYear)):
                if not GBEqual(ChildMortalityRates[DP_Data, a, b, t], ChildMortalityRates[DP_Default, a, b, t]):
                    DDC['ChildARTMort'] = False


    DDC['AdultProg'] = True
    AdultAnnRateProgressLowerCD4 = modvars[AM_AdultAnnRateProgressLowerCD4Tag] 
    for s in GBRange(GB_Male, GB_Female):
        for a in GBRange(DP_CD4_15_24, DP_CD4_45_54):
            for c in GBRange(DP_CD4_GT500, DP_CD4_50_99):
                if not GBEqual(AdultAnnRateProgressLowerCD4[DP_Data, s, a, c], AdultAnnRateProgressLowerCD4[DP_Default, s, a, c]):
                    DDC['AdultProg'] = False


    DDC['AdultDist'] = True
    AdultDistNewInfectionsCD4 = modvars[AM_AdultDistNewInfectionsCD4Tag] 
    for s in GBRange(GB_Male, GB_Female):
        for a in GBRange(DP_CD4_15_24, DP_CD4_45_54):
            for c in GBRange(DP_CD4_GT500, DP_CD4_LT50):
                if not GBEqual(AdultDistNewInfectionsCD4[DP_Data, s, a, c], AdultDistNewInfectionsCD4[DP_Default, s, a, c]):
                    DDC['AdultDist'] = False


    DDC['AdultMort'] = True
    AdultMortByCD4NoART = modvars[AM_AdultMortByCD4NoARTTag] 
    for s in GBRange(GB_Male, GB_Female):
        for a in GBRange(DP_CD4_15_24, DP_CD4_45_54):
            for c in GBRange(DP_CD4_GT500, DP_CD4_LT50):
                if not GBEqual(AdultMortByCD4NoART[DP_Data, s, a, c], AdultMortByCD4NoART[DP_Default, s, a, c]):
                    DDC['AdultMort'] = False


    DDC['AdultARTMort'] = True
    AdultMortByCD4WithART0to6 = modvars[AM_AdultMortByCD4WithART0to6Tag] 
    AdultMortByCD4WithART7to12 = modvars[AM_AdultMortByCD4WithART7to12Tag] 
    AdultMortByCD4WithARTGT12 = modvars[AM_AdultMortByCD4WithARTGT12Tag] 
    for s in GBRange(GB_Male, GB_Female):
        for a in GBRange(DP_CD4_15_24, DP_CD4_45_54):
            for c in GBRange(DP_CD4_GT500, DP_CD4_LT50):
                if not GBEqual(AdultMortByCD4WithART0to6[DP_Data, s, a, c], AdultMortByCD4WithART0to6[DP_Default, s, a, c]):
                    DDC['AdultARTMort'] = False
                if not GBEqual(AdultMortByCD4WithART7to12[DP_Data, s, a, c], AdultMortByCD4WithART7to12[DP_Default, s, a, c]):
                    DDC['AdultARTMort'] = False
                if not GBEqual(AdultMortByCD4WithARTGT12[DP_Data, s, a, c], AdultMortByCD4WithARTGT12[DP_Default, s, a, c]):
                    DDC['AdultARTMort'] = False

    MortalityRates = modvars[AM_MortalityRatesTag] 
    for b in GBRange(DP_MortRates_LT12Mo, DP_MortRates_GT12Mo):
        for t in GBRange(getYearIdx(firstYear, firstYear), getYearIdx(finalYear, firstYear)):
            if not GBEqual(MortalityRates[DP_Data, b, t], MortalityRates[DP_Default, b, t]):
                DDC['AdultARTMort'] = False


    DDC['FRRCD4'] = True
    FertCD4Discount = modvars[AM_FertCD4DiscountTag] 
    for c in GBRange(DP_CD4_GT500, DP_CD4_LT50):
        if not GBEqual(FertCD4Discount[DP_Data, c], FertCD4Discount[DP_Default, c]):
            DDC['FRRCD4'] = False


    DDC['FRRAge'] = True
    HIVTFR = modvars[AM_HIVTFRTag] 
    for a in GBRange(DP_A20_24, DP_A45_49):
        if not GBEqual(HIVTFR[DP_Data, a, yearIdx], HIVTFR[DP_Default, a, yearIdx]):
            DDC['FRRAge'] = False

    TFR15_19DefaultData = modvars[AM_TFR15_19DefaultDataTag] 
    TFRRegionMeta = modvars[AM_TFRRegion_MetaTag]['default'] 
    if not GBEqual(TFR15_19DefaultData[TFRRegionMeta], HIVTFR[DP_Default, DP_A15_19, yearIdx]):
        DDC['FRRAge'] = False


    DDC['CdArt'] = True
    RatioWomenOnART = modvars[AM_RatioWomenOnARTTag] 
    for a in GBRange(DP_A15_19, DP_A45_49):
        if not GBEqual(RatioWomenOnART[DP_Data, a], RatioWomenOnART[DP_Default, a]):
            DDC['CdArt'] = False


    DDC['MTCTRate'] = True
    TransEffAssump = modvars[AM_TransEffAssumpTag] 
    for regimen in GBRange(DP_NoProphExistInfCD4LT200, DP_ARTStartDurPreg_Late):
        for stage in GBRange(DP_Perinatal, DP_BreastfeedingGE350):
            if not GBEqual(TransEffAssump[DP_Data, regimen, stage], TransEffAssump[DP_Default, regimen, stage]):
                DDC['MTCTRate'] = False


    DDC['AllocDef'] = True
    NewARTPatAlloc = modvars[AM_NewARTPatAllocTag] 
    NewARTPatAllocMeta = modvars[AM_NewARTPatAlloc_MetaTag]['default']      
    for i in GBRange(DP_AdvOpt_ART_ExpMort, DP_AdvOpt_ART_PropElig):
        if not GBEqual(NewARTPatAlloc[i], NewARTPatAllocMeta[i]):
            DDC['AllocDef'] = False


    Shiny90AIDSDeathsFYear = modvars[AM_Shiny90AIDSDeathsFYearTag] 
    AIDSDeaths = modvars[AM_AIDSDeathsByAgeTag] 

    # AIDSDeathsSum = 0
    # for a in GBRange(0, 80):
    AIDSDeathsSum = AIDSDeaths[GB_BothSexes, 0:80, getYearIdx(finalYear, firstYear)].sum()
    DDC['Shiny90Valid'] = GBEqual(math.trunc(AIDSDeathsSum), math.trunc(Shiny90AIDSDeathsFYear))


    AIDSDeathsFYearUA = modvars[DP_AIDSDeathsFYearUATag] 
    AIDSDeaths = modvars[AM_AIDSDeathsByAgeTag] 

    # AIDSDeathsSum = 0
    # for a in GBRange(0, 80):
    AIDSDeathsSum = AIDSDeaths[GB_BothSexes, 0:80, getYearIdx(finalYear, firstYear)].sum()
    DDC['UAValid'] = GBEqual(math.trunc(AIDSDeathsSum), math.trunc(int(AIDSDeathsFYearUA)))


    DDC['AdultARTCov'] = True
    HAARTBySex = modvars[AM_HAARTBySexTag]  
    for s in GBRange(GB_Male, GB_Female):
        for t in GBRange(getYearIdx(firstYear, firstYear), getYearIdx(finalYear, firstYear)):
            if HAARTBySex[s, t] > GetAM_HIV(modvars, t, GB_BothSexes, 15, 80):
                DDC['AdultARTCov'] = False


    DDC['ChildARTCov'] = True
    for t in GBRange(getYearIdx(firstYear, firstYear), getYearIdx(finalYear, firstYear)):
        if getChildNumRecART(modvars, t) > GetAM_HIV(modvars, t, GB_BothSexes, 0, 14):
            DDC['ChildARTCov'] = False


    ChildOnPMTCT = modvars[AM_ChildOnPMTCTTag] 
    ChildNeedPMTCT = modvars[AM_ChildNeedPMTCTTag] 
    for t in GBRange(getYearIdx(firstYear, firstYear), getYearIdx(finalYear, firstYear)):
        if ChildOnPMTCT[t] > ChildNeedPMTCT[t]:
            DDC['PMTCTCov'] = False

    # DDC['EPPMaxAdj'] = (modvars[AM_IncidenceOptionsTag] == DP_EPPIncOpt) and (modvars[AM_EPPMaxAdjFactorTag] == 10)
    DDC['EPPMaxAdj'] = modvars[AM_EPPPrevAdjTag] and GBEqual(modvars[AM_EPPMaxAdjFactorTag], 10)

    return DDC



from .goveqs_basis2 import goveqs_basis2

def goveqs_scaleup(input, t, r, M0, M1, times, i, prm, sel, agg):
    scale = min(max((t-times[0])/(times[1]-times[0]),0),1) # % <--- NEW: removed min(,1), so that model can continue extrapolating ART coverage into future
    Mt = M1.copy()
    Mt['lin'] = M0['lin'] + scale*(M1['lin']-M0['lin'])
    Mt['Dxlin'] =  M0['Dxlin'] + scale*(M1['Dxlin'] - M0['Dxlin']) #Mt.Dxlin =  M0.Dxlin + scale*(M1.Dxlin - M0.Dxlin);
    Mt['linHIV'] = M0['linHIV'] + scale*(M1['linHIV'] - M0['linHIV']) #Mt['linHIV'] = M0.linHIV + scale*(M1.linHIV - M0.linHIV);
    Mt['linLTBI'] = M0['linLTBI'] + scale*(M1['linLTBI'] - M0['linLTBI']) #Mt.linLTBI = M0.linLTBI + scale*(M1.linLTBI - M0.linLTBI);

    out = goveqs_basis2(input, t, r, Mt, i, prm, sel, agg)
    return out




import math
from scipy import integrate

def ode15s(fun, init, steps, args):
    #fin_result = integrate.odeint(fun, init, steps, args, atol=1e-10, rtol=1e-10)
    fin_result = integrate.odeint(fun, init, steps, args, atol=1e-6, rtol=1e-6)
    #fin_result = integrate.odeint(fun, init, steps, args)
    return steps, fin_result

def create_steps(first, final, step):
    result = []
    i = first
    while((i<final) and (math.isclose(i, final)==False)):
        result.append(i)
        i = i+step
    result.append(final)
    return result




import numpy as np


from SpectrumCommon.Const.TB import *
from SpectrumCommon.Const.GB import *
from SpectrumCommon.Const.DP import *

def TB_ARTCohortsDefaults(pop, base_year, final_year):
    

    dur_slice = slice(DP_D_ARTlt6m-1, DP_D_ARTgt12m)
    cd4_slice = slice(DP_CD4_GT500-1, DP_CD4_LT50)
    dim_sum   = (1,2,3,4)
    total = [
            pop[:, : , 10:15, cd4_slice, dur_slice].sum(axis=dim_sum),   # 0-9
            pop[:, : , 10:15, cd4_slice, dur_slice].sum(axis=dim_sum),   # 10-14
            pop[:, : , 15:,       cd4_slice, dur_slice].sum(axis=dim_sum)    # 15 plus 
    ]

    # dur_slice = slice(DP_D_ARTlt6m-1, DP_D_ART6to12m)
    # cd4_slice = slice(DP_CD4_100_199-1, DP_CD4_LT50)
    ART_with_serious_illnes = [
        np.full(len(total[0]), 0.05), #         pop[:, : , 10:15, cd4_slice, dur_slice].sum(axis=dim_sum)/total[TB_ART10_14],   # 0-9
        np.full(len(total[0]), 0.05), #         pop[:, : , 10:15, cd4_slice, dur_slice].sum(axis=dim_sum)/total[TB_ART10_14], # 10-14
        np.full(len(total[0]), 0.05) #         pop[:, : , 15:,       cd4_slice, dur_slice].sum(axis=dim_sum)/total[TB_ART15p]    # 15 plus 
    ]

    # ART_with_serious_illnes = np.full((3, final_year-base_year), 0.05)
    # ART_without_serious_illnes = np.full((3, final_year-base_year), 0.95)
    # cd4_slice = slice(DP_CD4_GT500-1, DP_CD4_200_249)
    ART_without_serious_illnes = [
            1-ART_with_serious_illnes[TB_ART0_9],   # 0-9
            1-ART_with_serious_illnes[TB_ART10_14], # 10-14
            1-ART_with_serious_illnes[TB_ART15p]    # 15 plus
        
    ]
    art_cohort = [[total[a].tolist(), ART_without_serious_illnes[a].tolist(), ART_with_serious_illnes[a].tolist()] for a in range(TB_ART0_9, TB_ART15p+1)]
    
        # pop_15_plus = 0
        # ART_without_serious_illnes = [0]*3
        # ART_with_serious_illnes = [0]*3

    if final_year>2030:
        for yr in range(2030+1, final_year+1):
            art_cohort[0][0].append(art_cohort[0][0][-1])
            art_cohort[0][1].append(art_cohort[0][1][-1])
            art_cohort[0][2].append(art_cohort[0][2][-1])
            art_cohort[1][0].append(art_cohort[1][0][-1])
            art_cohort[1][1].append(art_cohort[1][1][-1])
            art_cohort[1][2].append(art_cohort[1][2][-1])
            art_cohort[2][0].append(art_cohort[2][0][-1])
            art_cohort[2][1].append(art_cohort[2][1][-1])
            art_cohort[2][2].append(art_cohort[2][2][-1])

    return np.nan_to_num(art_cohort, copy=True, nan=0.0, posinf=None, neginf=None)

def TB_ARTCharacteristicDefaults(c_ep, year_len):

    art_characteristics = np.zeros((TB_ARTAgeLen, TB_ART_Charac_len, year_len))   #35, 10, 100-c_ep
    art_characteristics[TB_ART0_9   ][TB_ART_WithTBInfection] += 0.35
    art_characteristics[TB_ART10_14 ][TB_ART_WithTBInfection] += 0.35
    art_characteristics[TB_ART15p   ][TB_ART_WithTBInfection] += 0.35  

    art_characteristics[TB_ART0_9   ][TB_ART_WithTBDisease] += 0.01
    art_characteristics[TB_ART10_14 ][TB_ART_WithTBDisease] += 0.01
    art_characteristics[TB_ART15p   ][TB_ART_WithTBDisease] += 0.01

    art_characteristics[TB_ART0_9   ][TB_ART_PercPTB] +=  1-c_ep
    art_characteristics[TB_ART10_14 ][TB_ART_PercPTB] +=  1-c_ep
    art_characteristics[TB_ART15p   ][TB_ART_PercPTB] +=  1-c_ep
    return art_characteristics


import SpectrumCommon.Const.LG.LGConst as lgc
import SpectrumCommon.Const.GB.GBConst as gbc

from AvenirCommon.Database import GB_get_db_json
from os import environ

#######################################################################################################
#                                                                                                     #
#                                       Cold chain                                                    #
#                                                                                                     #
#######################################################################################################


def LG_load_cold_chain_DB():
    cold_chain_DB_file_name = lgc.LG_COLD_CHAIN_DB_NAME + "_" + lgc.LG_COLD_CHAIN_DB_CURR_VERSION + "." + gbc.GB_JSON
    cold_chain_DB = GB_get_db_json(
        environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_LG_CONTAINER, cold_chain_DB_file_name
    )

    return cold_chain_DB



import datetime

from AvenirCommon.Database import *
from AvenirCommon.Util import GBGetKeyValue

from SpectrumCommon.Const.GB import *
from SpectrumCommon.Modvars.GB.GBDefs import *
from SpectrumCommon.Modvars.GB.GBUtil import get_country_details

from SpectrumCommon.Modvars.FP.FPModvars import init_modvars as init_FP_modvars
from SpectrumCommon.Modvars.TB.TBModvars import init_modvars as init_TB_modvars
from SpectrumCommon.Modvars.DP.DPModvars import init_modvars as init_DP_modvars
from SpectrumCommon.Modvars.AM.AMModvars import init_modvars as init_AM_modvars
from SpectrumCommon.Modvars.RP.RPModVars import init_modvars as init_RP_modvars
from SpectrumCommon.Modvars.CS.CSModvars import init_modvars as init_CS_modvars
from SpectrumCommon.Modvars.GB.GBModvars import init_modvars as init_GB_modvars
from SpectrumCommon.Modvars.VW.VWModvars import init_modvars as init_VW_modvars
from SpectrumCommon.Modvars.GB.PJNModvars import init_modvars as init_PJN_modvars
# IHT
from SpectrumCommon.Modvars.IH.IHModVars import init_modvars as init_IH_modvars
from SpectrumCommon.Modvars.IC.ICModVars import init_modvars as init_IC_modvars
from SpectrumCommon.Modvars.PC.PCModVars import init_modvars as init_PC_modvars
from SpectrumCommon.Modvars.HW.HWModVars import init_modvars as init_HW_modvars
from SpectrumCommon.Modvars.IS.ISModVars import init_modvars as init_IS_modvars
from SpectrumCommon.Modvars.HS.HSModVars import init_modvars as init_HS_modvars
from SpectrumCommon.Modvars.FS.FSModvars import init_modvars as init_FS_modvars
from SpectrumCommon.Modvars.BG.BGModVars import init_modvars as init_BG_modvars
from SpectrumCommon.Modvars.GV.GVModVars import init_modvars as init_GV_modvars
from SpectrumCommon.Modvars.HF.HFModVars import init_modvars as init_HF_modvars
from SpectrumCommon.Modvars.LG.LGModVars import init_modvars as init_LG_modvars
from SpectrumCommon.Modvars.HV.HVModvars import init_modvars as init_HV_modvars
from SpectrumCommon.Modvars.RN.RNModvars import init_modvars as init_RN_modvars

import SpectrumCommon.Const.IH.IHTags as ihmvt
import SpectrumCommon.Const.IC.ICTags as icmvt
import SpectrumCommon.Const.IS.ISTags as ismvt
import SpectrumCommon.Const.HW.HWTags as hwmvt
import SpectrumCommon.Const.PC.PCTags as pcmvt
import SpectrumCommon.Const.FP.FPTags as fpmvt
import SpectrumCommon.Const.CS.CSTags as csmvt

# from SpectrumCommon.Modvars.RP.RPModVarDefs import RP_init_sector_list

#from App.TempCalc.GB.GBMain import GBCalculate

# def get_country_ISO3Alpha(countryISO3 = int):
#     countries = GB_get_db_json(env[GB_SPECT_MOD_DATA_CONN_ENV], 'globals', formatCountryFName(GBCountryListDBName, GBDatabaseVersion))
#     for country in countries:
#         if country['ISO3_Numeric'] == countryISO3:
#             return country['ISO3_Alpha']

# Sorts the modules if we need something from one when initializing another.
def sort_modules(modules : list):
    modules_sorted = []

    # Add modules whose order we care about first:
    #   PC needs programme areas from IH.
    #   HW needs facility types from IS.
    #   IC needs programme areas from IH, staff types from HW, costing sections from PC, active methods from FamPlan, selected interventions from LiST.
    #   BG needs many modules; IH, IC, PC, HR, IS, HS, HW, GV, HF, LG
    #   RP needs FP
    if GB_IH in modules:
        modules_sorted.append(GB_IH)
    if GB_PC in modules:
        modules_sorted.append(GB_PC)
    if GB_IS in modules:
        modules_sorted.append(GB_IS)
    if GB_HW in modules:
        modules_sorted.append(GB_HW)
    if GB_HS in modules:
        modules_sorted.append(GB_HS)
    if GB_GV in modules:
        modules_sorted.append(GB_GV)
    if GB_HF in modules:
        modules_sorted.append(GB_HF)
    if GB_FP in modules:
        modules_sorted.append(GB_FP)
    if GB_RP in modules:
        modules_sorted.append(GB_RP)
    if GB_CS in modules:
        modules_sorted.append(GB_CS)
    if GB_IC in modules:
        modules_sorted.append(GB_IC)
    if GB_FS in modules:
        modules_sorted.append(GB_FS)
    if GB_BG in modules:
        modules_sorted.append(GB_BG)
    # if GB_HV in modules:
    #     modules_sorted.append(GB_HV)
    # if GB_RN in modules:
    #     modules_sorted.append(GB_RN)
    
    # Get list of the modules we haven't added yet.
    modules_diff = list(set(modules) - set(modules_sorted))

    # Add other modules whose order we don't care about.
    for module in modules_diff:
        if module not in GB_PseudoModules:
            modules_sorted.append(module)

    return modules_sorted

def GB_CreateProjection(params : createProjectionParams):

        #convert module strings to id's
        if not params.firstYear:
            date = datetime.datetime.now().date()
            params.firstYear = int(date.strftime("%Y"))
        countryID = params.country
        first_year = params.firstYear
        final_year = params.finalYear

        modules = []
        for md in params.modules:
            if type(md)==str:
                md = md.strip()
            if isinstance(md, str):
                if md in GB_MOD_ID:
                    modules.append(GB_MOD_ID[md])
            else:
                 if md in GB_EXISTING_MOD_ID.values():
                    modules.append(md)
        params.modules = modules

        # Extra params

        extra = params.extra
        if GB_TB_COSTING_ON not in extra:
            extra[GB_TB_COSTING_ON] = False

        if GB_LIST_COSTING_ON not in extra:
            extra[GB_LIST_COSTING_ON] = False

        if GB_IHT_ON not in extra:
            extra[GB_IHT_ON] = False

        # extra = params.extra
        # if not (GB_RAPID_SECTORS_ON in extra):
        #     extra[GB_RAPID_SECTORS_ON] = RP_init_sector_list()

        subNatCode = GBGetKeyValue(extra, 'subNatCode', 0)
        calculate = GBGetKeyValue(extra, 'calculate', 0)

        extra['fileName'] = params.fileName
        extra[GB_MODULES] = params.modules

        extra['countryDetails'] = get_country_details(countryID, subNatCode)

        if extra['countryDetails'] == None:
            raise Exception('Country ' + countryID + ' does not exist')

        init_modvars_fun = {
            GB_GB : init_GB_modvars,
            GB_PJ : init_PJN_modvars,
            GB_DP : init_DP_modvars,
            GB_AM : init_AM_modvars,
            GB_FP : init_FP_modvars,
            GB_RP : init_RP_modvars,
            GB_TB : init_TB_modvars,
            GB_CS : init_CS_modvars,
            GB_IH : init_IH_modvars,
            GB_IC : init_IC_modvars,
            GB_PC : init_PC_modvars,
            GB_HW : init_HW_modvars,
            GB_IS : init_IS_modvars,
            GB_HS : init_HS_modvars,
            GB_FS : init_FS_modvars,
            GB_BG : init_BG_modvars,
            GB_GV : init_GV_modvars,
            GB_HF : init_HF_modvars,
            GB_LG : init_LG_modvars,
            GB_HV : init_HV_modvars,
            GB_RN : init_RN_modvars,
            # GB_UA : init_UA_modvars,
        }

        #force GB, PJ, and DP to be in modules
        modules.append(GB_GB) if GB_GB not in modules else None
        modules.append(GB_PJ) if GB_PJ not in modules else None
        modules.append(GB_DP) if GB_DP not in modules else None

        # Add IH for modules that require it
        if (GB_IH not in modules) and ((GB_IC in modules) or (GB_PC in modules)):
            modules.append(GB_IH)

        if (GB_IS not in modules) and (GB_HW in modules):
            modules.append(GB_IS)

        if (GB_HW not in modules) and (GB_IC in modules):
            modules.append(GB_HW)

        if (GB_PC not in modules) and (GB_IC in modules):
            modules.append(GB_PC)

        # Temporarily require BG for TB app until we get modify projection working.
        if (GB_BG not in modules) and extra[GB_TB_COSTING_ON]:
            modules.append(GB_BG)

        modules_sorted = sort_modules(modules)

        #all_modvars = {}
        projection = {}#init_PJN_modvars(countryID, first_year, final_year, extra)

        # MVs needed for PC, IC; comes from IH
        prog_areas = None
        health_sys_admin_units = None
        # MVs needed for HW; comes from IS
        facilities = None
        # MVs needed for IC; comes from HW, PC, FP, CS respectively
        staff_types = None
        costing_sections = None
        activities = None
        active_methods = None # Also needed for RP
        selected_intervs_CS = None
        IVInfo_CS = None
        syph_among_WRA = None
        enter_BF_improve_data = None
        # MVs needed for BG; comes from IC, IH, respectively
        drugs_supplies = None
        interventions = None
        deliv_chans = None
        # MVs needed for RP
        method_mix = None

        for m in modules_sorted:
            if m in init_modvars_fun:
                try:
                    extra_info = None

                    # If we're on PC, add ModVars from IH that it needs
                    if m == GB_PC:
                        extra_info = {
                            ihmvt.IH_PROG_AREA_RECORDS_TAG_V1 : prog_areas,
                            ihmvt.IH_HEALTH_SYS_ADMIN_UNIT_RECORDS_TAG : health_sys_admin_units
                        }
                        extra_info.update(extra)

                    elif m == GB_IS:
                        extra_info = {
                            ihmvt.IH_PROG_AREA_RECORDS_TAG_V1 : prog_areas,
                        }
                        extra_info.update(extra)

                    elif m == GB_HW:
                        extra_info = {ismvt.IS_FACILITY_TYPE_RECORDS_TAG : facilities}
                        extra_info.update(extra)

                    elif m == GB_IC:
                        extra_info = {
                            ihmvt.IH_PROG_AREA_RECORDS_TAG_V1    : prog_areas,
                            hwmvt.HW_STAFF_TYPE_RECORDS_TAG_V1   : staff_types,
                            pcmvt.PC_COSTING_SECT_RECORDS_TAG    : costing_sections,
                            fpmvt.FP_ActiveMethodsTag            : active_methods,
                            csmvt.CS_SelectedIVSetTag            : selected_intervs_CS,
                            csmvt.CS_IVInfoTag                   : IVInfo_CS,
                            csmvt.CS_SyphilisAmongWRATag         : syph_among_WRA,
                            # csmvt.CS_EnterBFImproveDataTag       : enter_BF_improve_data,
                        }
                        extra_info.update(extra)

                    elif m == GB_BG:
                        extra_info = {
                            icmvt.IC_INTERV_RECORDS_TAG           : interventions,
                            icmvt.IC_DRUG_SUPPLY_RECORDS_TAG      : drugs_supplies,
                            ihmvt.IH_PROG_AREA_RECORDS_TAG_V1     : prog_areas,
                            ihmvt.IH_DELIV_CHAN_RECORDS_TAG_V1    : deliv_chans,
                            pcmvt.PC_COSTING_SECT_RECORDS_TAG     : costing_sections,
                            pcmvt.PC_ACT_RECORDS_TAG              : activities,
                            ismvt.IS_FACILITY_TYPE_RECORDS_TAG    : facilities,
                            hwmvt.HW_STAFF_TYPE_RECORDS_TAG_V1    : staff_types,
                        }
                        extra_info.update(extra)

                    elif m == GB_CS:
                        #Give these fields in extra default values in case they were not initialized.
                        def init_extra_field(key, value):
                            if key not in extra:
                                extra[key] = value

                        init_extra_field(GB_GSKMode, False)
                        init_extra_field('exploreLiST', False)
                        init_extra_field('level', '')
                        init_extra_field('source', '')
                        init_extra_field('appID', '')

                        extra_info = {}
                        extra_info.update(extra)

                    elif m == GB_FP:
                        #Give these fields in extra default values in case they were not initialized.
                        def init_extra_field(key, value):
                            if key not in extra:
                                extra[key] = value

                        init_extra_field('exploreLiST', False)
                        init_extra_field('RapidProj', GB_RP in modules)
                        init_extra_field('LiSTProj', GB_CS in modules)

                        extra_info = {}
                        extra_info.update(extra)

                    elif m == GB_RP:
                        extra_info = {
                            fpmvt.FP_ActiveMethodsTag : active_methods,
                            fpmvt.FP_MethodMixTag     : method_mix,
                        }
                        extra_info.update(extra)

                    else:
                        extra_info = extra

                    modvars = init_modvars_fun[m](countryID, first_year, final_year, extra_info)

                    # Save MVs needed in dependent modules
                    if m == GB_IH:
                        prog_areas = modvars[ihmvt.IH_PROG_AREA_RECORDS_TAG_V1]
                        deliv_chans = modvars[ihmvt.IH_DELIV_CHAN_RECORDS_TAG_V1]
                        health_sys_admin_units = modvars[ihmvt.IH_HEALTH_SYS_ADMIN_UNIT_RECORDS_TAG]

                    elif m == GB_IS:
                        facilities = modvars[ismvt.IS_FACILITY_TYPE_RECORDS_TAG]

                    elif m == GB_HW:
                        staff_types = modvars[hwmvt.HW_STAFF_TYPE_RECORDS_TAG_V1]

                    elif m == GB_PC:
                        costing_sections = modvars[pcmvt.PC_COSTING_SECT_RECORDS_TAG]
                        activities = modvars[pcmvt.PC_ACT_RECORDS_TAG]

                    elif m == GB_IC:
                        interventions = modvars[icmvt.IC_INTERV_RECORDS_TAG]
                        drugs_supplies = modvars[icmvt.IC_DRUG_SUPPLY_RECORDS_TAG]

                    elif m == GB_CS:
                        selected_intervs_CS = modvars[csmvt.CS_SelectedIVSetTag]
                        IVInfo_CS = modvars[csmvt.CS_IVInfoTag]
                        syph_among_WRA = modvars[csmvt.CS_SyphilisAmongWRATag]
                        # enter_BF_improve_data = modvars[csmvt.CS_EnterBFImproveDataTag]

                    elif m == GB_FP:
                        active_methods = modvars[fpmvt.FP_ActiveMethodsTag]
                        method_mix = modvars[fpmvt.FP_MethodMixTag]

                    projection.update(modvars)
                except Exception as e:
                    raise Exception({"msg": f"Failed to create {GB_MOD_STR[m]}",
                                     "error" : e.args[0] if len(e.args)>0 else ''})

            else:
                raise Exception(f"Module {m} does not exist")

#        for tag in projection:
#            try:
#                mv_jsonstr = ujson.dumps(projection[tag])
#            except:
#                logger.error(f'Failed to JSONIFY {tag}')

        if not extra_info.get("level","") == "":
            modvars = init_VW_modvars(countryID, first_year, final_year, extra_info, projection)
            projection.update(modvars)
            params.modules.append(GB_VW)

        return projection



def InterventionInit(): #function [Intvn]=InterventionInit()

    Intvn={} #Intvn=[];
    Intvn['year1']=[] #Intvn['year1']=[];
    Intvn['year2']=[] #Intvn.year2=[];
    Intvn['year1_vac']=[] #Intvn.year1_vac=[];
    Intvn['year2_vac']=[] #Intvn.year2_vac=[];

    Intvn['r_cs']=[] #Intvn.r_cs=[];           
    Intvn['r_cs2']=[] #Intvn.r_cs2=[];          
    Intvn['r_cs3']=[] #Intvn.r_cs3=[];  

    Intvn['p_Dx']=[] #Intvn.p_Dx=[];  

    Intvn['p_MDRrec']=[] #Intvn.p_MDRrec=[]; 
    Intvn['p_cure']=[] #Intvn.p_cure=[]; 
    Intvn['r_default']=[] #Intvn.r_default=[]; 
    Intvn['r_default2']=[] #; 

    Intvn['r_progression']=[] #Intvn.r_progression=[];	
    Intvn['r_reactivation']=[] #Intvn.r_reactivation=[]; 

    Intvn['p_pu']=[] #Intvn.p_pu=[];                  
    Intvn['r_Tx2']=[]#Intvn.r_Tx2=[];          
    Intvn['p_cure2']=[]#Intvn.p_cure2=[];        
    Intvn['p_TX_complete']=[] #Intvn.p_TX_complete=[]; 
    Intvn['r_relapse_1']=[] #Intvn.r_relapse_1=[];	
    Intvn['r_relapse_2']=[] #Intvn.r_relapse_2=[];   
    Intvn['r_relapse_3']=[] #Intvn.r_relapse_3=[]; 	
    Intvn['p_VE_1']=[] #Intvn.p_VE_1=[];        
    Intvn['p_VE_2']=[] #Intvn.p_VE_2=[];      
    Intvn['r_vacc']=[] #Intvn.r_vacc=[];        
    Intvn['r_waning']=[] #Intvn.r_waning=[]; 

    return(Intvn)